<?php

namespace IPS\videobox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Editor Extension: Icons
 */
class _IMG
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'videobox_videos', array( 'v_thumbnail<>? AND v_thumbnail IS NOT NULL', '' ) )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$video = \IPS\Db::i()->select( '*', 'videobox_videos', "v_thumbnail IS NOT NULL AND v_thumbnail != ''", 'v_id', array( $offset, 1 ) )->first();
		
		try
		{		
			$thumbnail			= \IPS\File::get( $oldConfiguration ?: 'videobox_IMG', $video['v_thumbnail'] )->move( $storageConfiguration );
			$thumbnail_medium	= \IPS\File::get( $oldConfiguration ?: 'videobox_IMG', $video['v_thumbnail_medium'] )->move( $storageConfiguration );	
			
			\IPS\Db::i()->update( 'videobox_videos', array( 'v_thumbnail' 			=> (string) $thumbnail,
															'v_thumbnail_medium' 	=> (string) $thumbnail_medium ), array( 'v_id=?', $video['v_id'] ) );		
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}		
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$video = \IPS\Db::i()->select( '*', 'videobox_videos', "v_thumbnail IS NOT NULL AND v_thumbnail != ''", 'v_id', array( $offset, 1 ) )->first();
		if ( $new = \IPS\File::repairUrl( $video['v_thumbnail'] ) )
		{
			\IPS\Db::i()->update( 'videobox_videos', array( 'v_thumbnail' => $new ), array( 'v_id=?', $video['v_id'] ) );
		}
		
		if ( $video['v_thumbnail_medium'] and $new = \IPS\File::repairUrl( $video['v_thumbnail_medium'] ) )
		{
			\IPS\Db::i()->update( 'videobox_videos', array( 'v_thumbnail_medium' => $new ), array( 'v_id=?', $video['v_id'] ) );
		}	
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'videobox_videos', array( 'v_thumbnail=? OR v_thumbnail_medium=?', $file, $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'videobox_videos', "v_thumbnail IS NOT NULL AND v_thumbnail != ''" ) as $video )
		{
			try
			{
				\IPS\File::get( 'videobox_IMG', $video['v_thumbnail'] )->delete();
				\IPS\File::get( 'videobox_IMG', $video['v_thumbnail_medium'] )->delete();
			}
			catch( \Exception $e ){}
		}
	}
}