<?php

namespace IPS\videobox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Editor Extension: Icons
 */
class _Slider
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'videobox_videos', "v_featuredIMG IS NOT NULL AND v_featuredIMG != ''" )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$video = \IPS\Db::i()->select( '*', 'videobox_videos', "v_featuredIMG IS NOT NULL AND v_featuredIMG != ''", 'v_id', array( $offset, 1 ) )->first();
		
		try
		{
			$image = \IPS\File::get( $oldConfiguration ?: 'videobox_Slider', $video['v_featuredIMG'] )->move( $storageConfiguration );
			$org = \IPS\File::get( $oldConfiguration ?: 'videobox_Slider', $video['v_featuredIMG_original'] )->move( $storageConfiguration );
			
			\IPS\Db::i()->update( 'videobox_videos', array( 'v_featuredIMG' => (string) $image, 'v_featuredIMG_original' => (string) $org ), array( 'v_id=?', $video['v_id'] ) );	
		}
		catch( \Exception $e ) {}		
	}
	
	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$video = \IPS\Db::i()->select( '*', 'videobox_videos', "v_featuredIMG IS NOT NULL AND v_featuredIMG != ''", 'v_id', array( $offset, 1 ) )->first();
		if ( $new = \IPS\File::repairUrl( $video['v_featuredIMG_original'] ) )
		{
			\IPS\Db::i()->update( 'videobox_videos', array( 'v_featuredIMG_original' => $new ), array( 'v_id=?', $video['v_id'] ) );
		}
		
		if ( $video['v_featuredIMG'] and $new = \IPS\File::repairUrl( $video['v_featuredIMG'] ) )
		{
			\IPS\Db::i()->update( 'videobox_videos', array( 'v_featuredIMG' => $new ), array( 'v_id=?', $video['v_id'] ) );
		}
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'videobox_videos', array( 'v_featuredIMG=?', (string) $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'videobox_videos', "v_featuredIMG IS NOT NULL AND v_featuredIMG != ''" ) as $video )
		{
			try
			{
				\IPS\File::get( 'videobox_Slider', $video['v_featuredIMG_original'] )->delete();
				\IPS\File::get( 'videobox_Slider', $video['v_featuredIMG'] )->delete();
			}
			catch( \Exception $e ){}
		}
	}
}