<?php

namespace IPS\videobox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Editor Extension: Icons
 */
class _Videos
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'videobox_online', array( 'vbo_url<>? AND vbo_url IS NOT NULL AND vbo_upload=1', '' ) )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$video = \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_url<>? AND vbo_url IS NOT NULL AND vbo_upload=1', '' ), 'vbo_id', array( $offset, 1 ) )->first();
		
		try
		{			
			$file = \IPS\File::get( $oldConfiguration ?: 'videobox_Videos', $video['vbo_url'] )->move( $storageConfiguration );
			\IPS\Db::i()->update( 'videobox_online', array( 'vbo_url' => (string) $file ), array( 'vbo_id=?', $video['vbo_id'] ) );	
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}				
	}
	
	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$video = \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_url<>? AND vbo_url IS NOT NULL AND vbo_upload=1', '' ), 'vbo_id', array( $offset, 1 ) )->first();
		if ( $new = \IPS\File::repairUrl( $video['vbo_url'] ) )
		{
			\IPS\Db::i()->update( 'videobox_videos', array( 'vbo_url' => $new ), array( 'vbo_id=?', $video['vbo_id'] ) );
		}
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_url<>? AND vbo_url IS NOT NULL AND vbo_upload=1', '' ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_url<>? AND vbo_url IS NOT NULL AND vbo_upload=1', '' ) ) as $video )
		{
			try
			{
				\IPS\File::get( 'videobox_Videos', $video['vbo_url'] )->delete();			
			}
			catch( \Exception $e ){}
		}
	}
}