<?php
/**
 * @brief		Admin CP Group Form
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Videobox
 * @since		06 Oct 2017
 */

namespace IPS\videobox\extensions\core\GroupForm;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Admin CP Group Form
 */
class _Videobox
{
	/**
	 * Process Form
	 *
	 * @param	\IPS\Helpers\Form		$form	The form
	 * @param	\IPS\Member\Group		$group	Existing Group
	 * @return	void
	 */
	public function process( &$form, $group )
	{			
		$form->add( new \IPS\Helpers\Form\Number( 'videobox_limit_videos', $group->videobox_limit_videos, FALSE, array( 'unlimited' => 0, 'unlimitedLang' => 'videobox_limit_videos_unlimited' ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('videobox_limit_videos_suffix') ) );
	}
	
	/**
	 * Save
	 *
	 * @param	array				$values	Values from form
	 * @param	\IPS\Member\Group	$group	The group
	 * @return	void
	 */
	public function save( $values, &$group )
	{
		$limit_video = $values['videobox_limit_videos'];
		
		try
		{
			$vb_group = \IPS\Db::i()->select( '*', 'videobox_member_groups', array( 'vb_group_group=?', $group->g_id ) )->first();
			\IPS\Db::i()->update( 'videobox_member_groups', array( 'videobox_limit_videos' => $limit_video ), array( 'vb_group_group=?', $group->g_id ) );
		}
		catch( \UnderflowException $ex ) 
		{
			\IPS\Db::i()->insert( 'videobox_member_groups', array(
				'videobox_limit_videos'	=> $limit_video,
				'vb_group_group'		=> $group->g_id,
			) );			
		}	
	}
	
	
	
}