<?php
/**
 * @brief		Background Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Videobox
 * @since		11 Jan 2017
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\extensions\core\Queue;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Background Task
 */
class _vbUpdateTwitch
{
	/**
	 * @brief   Number of videos to rebuild per cycle
	 */
	public $rebuild	= 100;
	
	/**
	 * Parse data before queuing
	 *
	 * @param	array	$data
	 * @return	array
	 */
	public function preQueueData( $data )
	{
		return $data;
	}

	/**
	 * Run Background Task
	 *
	 * @param	mixed						$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int							$offset	Offset
	 * @return	int							New offset
	 * @throws	\IPS\Task\Queue\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function run( $data, $offset )
	{
		$where = "v_livestream LIKE '%twitch.tv%'";
		$select = \IPS\Db::i()->select( '*', 'videobox_videos', $where, 'v_id DESC', array( $offset, $this->rebuild ) );
		if ( !$select->count() )
		{
			throw new \IPS\Task\Queue\OutOfRangeException;
		}
		
		foreach( $select as $row )
		{
			$video = \IPS\videobox\Video::constructFromData( $row );
			
			$name = explode( '/', $video->livestream );
			$name = end( $name );
			$data = \IPS\Application::load('videobox')->requestStreamData( $video->livestream, $name );
	
			if ( $video->livestream_online == 'offline' && $data['online'] == 'offline' )
			{
				continue;
			}			
			$video->livestream_online = $data['online'];
			$video->livestream_data = @json_encode( $data );
			$video->livestream_updated = time();
			$video->save();			
		}

		return $offset + $this->rebuild;	
	}
	
	/**
	 * Get Progress
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	array( 'text' => 'Doing something...', 'complete' => 50 )	Text explaining task and percentage complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function getProgress( $data, $offset )
	{
		return array(
			'text'      => \IPS\Member::loggedIn()->language()->addToStack( 'vb_queue_twitch', FALSE ),
			'complete'  => ( isset( $data['count'] ) AND $data['count'] ) 
				? ( round( 100 / $data['count'] * $offset, 2 ) ) 
				: 100
		);	
	}
}