<?php


namespace IPS\videobox\modules\admin\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * mediatags
 */
class _mediatags extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'mediatags_manage' );
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'videobox_mediatags', \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags' ) );
		$table->langPrefix = 'videoboxMediatag_';

		/* Column stuff */
		$table->include = array( 'vbmt_name' );
		$table->mainColumn = 'vbmt_name';

		/* Sort stuff */
		$table->sortBy = $table->sortBy ?: 'vbmt_name';
		$table->sortDirection = $table->sortDirection ?: 'asc';

		/* Search */
		$table->quickSearch = 'vbmt_name';
		
		/* Root buttons */
		if ( \IPS\Member::loggedIn()->hasAcpRestriction( 'mediatag', 'mediatag', 'vbmt_add' ) )
		{
			$table->rootButtons = array(
				'add'		=> array(
					'icon'		=> 'plus',
					'title'		=> 'vbmt_add',
					'link'		=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags&do=mediatag' ),
					'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('vbmt_add') )
				)
			);
		}
		
		/* Row buttons */
		$table->rowButtons = function( $row )
		{
			$return = array();
			
			$return['edit'] = array(
				'icon'		=> 'pencil',
				'title'		=> 'edit',
				'link'		=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags&do=mediatag&id=' ) . $row['vbmt_id'],
				'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('edit') )
			);
			
			$return['delete'] = array(
				'icon'		=> 'times',
				'title'		=> 'delete',
				'link'		=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags&do=delete&id=' ) . $row['vbmt_id'],
				'data'		=> array( 'delete' => '' )
			);
			
			return $return;
		};

		/* Display */
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('vbmt_title');
		\IPS\Output::i()->output	= (string) $table;
	}
	
	/**
	 * Add/Edit Media Tag
	 */
	public function mediatag()
	{
		if ( \IPS\Request::i()->id )
		{
			try
			{
				$tag = \IPS\videobox\Mediatag::load( \IPS\Request::i()->id );
			}
			catch ( \UnderflowException $e )
			{
				\IPS\Output::i()->error( 'vbmt_not_found', '2BIMVB100/1', 404 );
			}
		}
		else
		{
			$tag = new \IPS\videobox\Mediatag;
		}
		
		/* form */
		$form = new \IPS\Helpers\Form;
		$form->add( new \IPS\Helpers\Form\Text( 'vbmt_name', ( \IPS\Request::i()->id ) ? $tag->name : NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Text( 'vbmt_match', ( \IPS\Request::i()->id ) ? $tag->match : NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\TextArea( 'vbmt_replace', ( \IPS\Request::i()->id ) ? $tag->replace : NULL, TRUE, array('rows' => 4) ) );

		if ( $values = $form->values() )
		{
			if ( $values['vbmt_name'] and $values['vbmt_match'] and $values['vbmt_replace'] )
			{
				$tag->name = $values['vbmt_name'];
				$tag->match	= $values['vbmt_match'];
				$tag->replace = $values['vbmt_replace'];
				$tag->save();

				\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags' ), 'saved' );
			}
			else
			{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack('vbmt_can_not_save');
			}
		}

		/* Output */
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('vbmt_add');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( \IPS\Output::i()->title, $form, FALSE );
	}
	
	/**
	 * Delete
	 */
	public function delete()
	{
		$tag = \IPS\videobox\Mediatag::load( \IPS\Request::i()->id );
		
		try
		{
			$tag->delete();
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'vbmt_not_found', '2BIMVB100/2', 404 );
		}

		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags' ), 'deleted' );
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('delete');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( \IPS\Output::i()->title, $form, FALSE );
	}	
}