<?php


namespace IPS\videobox\modules\admin\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * pages
 */
class _pages extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'pages_manage' );
		parent::execute();
	}

	protected function manage()
	{
		/* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'videobox_pages', \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages' ) );
		$table->langPrefix = '';

		/* Column stuff */
		$table->include = array( 'vbp_title', 'vbp_furl', 'vbp_updated' );
		$table->mainColumn = 'vbp_title';

		/* Sort stuff */
		$table->sortBy = $table->sortBy ?: 'vbp_id';
		$table->sortDirection = $table->sortDirection ?: 'desc';

		/* Search */
		$table->quickSearch = 'vbp_title';
		$table->advancedSearch = array(
			'vbp_title'		=> \IPS\Helpers\Table\SEARCH_CONTAINS_TEXT,
			'vbp_furl'		=> \IPS\Helpers\Table\SEARCH_CONTAINS_TEXT,
			'vbp_updated'	=> \IPS\Helpers\Table\SEARCH_DATE_RANGE,
		);

		/* Formatters */
		$table->parsers = array(
			'vbp_title'	=> function( $val, $row )
			{
				if ( \IPS\Member::loggedIn()->language()->checkKeyExists( 'videobox_page_' . $row['vbp_id'] ) )
				{
					return \IPS\Member::loggedIn()->language()->addToStack( 'videobox_page_' . $row['vbp_id'] );
				}
				else return $val;
			},				
			'vbp_furl'	=> function( $val, $row )
			{
				$url = \IPS\Http\Url::internal( "app=videobox&module=video&controller=page&page=" . $val, 'front', 'videobox_page', $val ); 
				return "<a href='{$url}' target=_blank>{$url}</a>";
			},		
			'vbp_updated'	=> function( $val, $row )
			{
				$date	= \IPS\DateTime::ts( $val );
				return $date->localeDate() . ' ' . $date->localeTime( FALSE ) ;
			}
		);

		/* Row buttons */
		$table->rowButtons = function( $row )
		{
			$return['offline']	= array(
				'icon'	=> $row['vbp_open'] == 1 ? 'toggle-on' : 'toggle-off', 
				'title'	=> $row['vbp_open'] == 1 ? 'turn_offline' : 'turn_online',
				'link'	=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages&do=enableToggle&id=' ) . $row['vbp_id'],
			);
			$return['edit'] = array(
				'icon'		=> 'pencil',
				'title'		=> 'edit',
				'link'		=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages&do=edit&id=' ) . $row['vbp_id'],
				'data'		=> array()
			);
			$return['delete'] = array(
				'icon'		=> 'times-circle',
				'title'		=> 'delete',
				'link'		=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages&do=delete&id=' ) . $row['vbp_id'],
				'data'		=> array( 'delete' => '' ),
			);
			return $return;
		};

		/* Root buttons */
		$table->rootButtons = array(
			'add'		=> array(
				'icon'		=> 'plus',
				'title'		=> 'videobox_page_add',
				'link'		=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages&do=add' ),
				'data'		=> array()
			),
		);
			
		/* Display */
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('videobox_pages');
		\IPS\Output::i()->output	= (string) $table;
	}
	
	/*-------------------------------------------------------------------------*/
	// Build the form
	/*-------------------------------------------------------------------------*/	
	protected function buildForm( $page )
	{
		/* Build Form */
		$form = new \IPS\Helpers\Form;

		$form->add( new \IPS\Helpers\Form\Translatable( 'vbp_title', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $page ? "videobox_page_{$page->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'vbp_furl', $page ? $page->furl : NULL , TRUE ) );		
		$form->add( new \IPS\Helpers\Form\Translatable( 'vbp_metakeywords', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $page ? "videobox_page_{$page->id}_metakeywords" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'vbp_metadescription', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $page ? "videobox_page_{$page->id}_metadescription" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Select( 'vbp_groups', $page ? array_filter( explode( ',', $page->groups ) ) : FALSE, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vbp_wrapper', $page ? $page->wrapper : 1, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'vbp_content', NULL, TRUE, array(
			'app'		=> 'videobox',
			'key'		=> ( $page->id ? "videobox_page_{$page->id}_content" : NULL ),
			'editor'	=> array(
				'app'			=> 'videobox',
				'key'			=> 'Pages',
				'autoSaveKey'	=> ( $page->id ? "videobox-page-{$page->id}" : "videobox-new-page" ),
				'attachIds'		=> $page->id ? array( $page->id ) : NULL
			)
		) ) );
		return $form;
	}
	
	/*-------------------------------------------------------------------------*/
	// Add new page
	/*-------------------------------------------------------------------------*/	
	public function add()
	{
		/* Build the form */
		$form = $this->buildForm( $page );

		/* Handle submissions */
		if ( $values = $form->values() )
		{
			/* Insert the new record */
			$page = new \IPS\videobox\Page;
			$this->savePageRecord( $page, $values );
			\IPS\File::claimAttachments( 'videobox-new-page', $page->id );
			
			/* Admin log */
			\IPS\Session::i()->log( 'acplog__videobox_pagecreated', array( $page->title => TRUE ) );

			/* Redirect */
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages' ), 'saved' );
		}
		
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('videobox_page_add');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'videobox_page_add', $form, FALSE );
	}

	/*-------------------------------------------------------------------------*/
	// Edit page
	/*-------------------------------------------------------------------------*/
	public function edit()
	{
		/* Load existing page */
		try
		{
			$page	= \IPS\videobox\Page::load( \IPS\Request::i()->id );
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_page_not_found', '2BIMVB101/1', 404 );
		}		
		
		/* Build the form */
		$form = $this->buildForm( $page );

		/* Handle submissions */
		if ( $values = $form->values() )
		{
			$this->savePageRecord( $page, $values );
			\IPS\File::claimAttachments( 'videobox-page-' . $page->id, $page->id );
			
			/* Admin log */
			\IPS\Session::i()->log( 'acplog__videobox_pageupdated', array( $page->title => TRUE ) );

			/* Redirect */
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages' ), 'saved' );
		}

		\IPS\Output::i()->title		= $page->title;
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( $page->title, $form, FALSE );
	}	
	
	/*-------------------------------------------------------------------------*/
	// Edit page
	/*-------------------------------------------------------------------------*/
	protected function savePageRecord( $page, $values )
	{
		# Save record
		$page->furl				= $values['vbp_furl'];
		$page->groups			= \is_array( $values['vbp_groups'] ) ? implode( ",", $values['vbp_groups'] ) : '';
		$page->wrapper			= $values['vbp_wrapper'];
		$page->updated			= time();
		$page->save();
		
		# Save lang
		$lang = array(
			'vbp_title' 			=> 'videobox_page_' . $page->id, 
			'vbp_metakeywords' 		=> 'videobox_page_' . $page->id . '_metakeywords',
			'vbp_metadescription' 	=> 'videobox_page_' . $page->id . '_metadescription',
			'vbp_content' 			=> 'videobox_page_' . $page->id . '_content',
		);
		foreach ( $lang as $fieldKey => $langKey )
		{
			if ( array_key_exists( $fieldKey, $values ) )
			{
				\IPS\Lang::saveCustom( 'videobox', $langKey, $values[ $fieldKey ] );
				unset( $values[ $fieldKey ] );
			}
		}	
	}
	
	/*-------------------------------------------------------------------------*/
	// Delete page
	/*-------------------------------------------------------------------------*/	
	public function delete()
	{
		/* Delete the page */
		try
		{
			$page	= \IPS\videobox\Page::load( \IPS\Request::i()->id );
			$page->delete();
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_page_not_found', '2BIMVB101/2', 404 );
		}
		
		/* Admin log */
		\IPS\Session::i()->log( 'acplog__videobox_pagedeleted', array( $page->title => TRUE ) );

		/* Redirect */
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages' ), 'deleted' );

		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('delete');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'delete', $form, FALSE );
	}	
	
	/*-------------------------------------------------------------------------*/
	// Toggle enable/disable
	/*-------------------------------------------------------------------------*/	
	public function enableToggle()
	{
		/* Toggle */
		try
		{
			$page	= \IPS\videobox\Page::load( \IPS\Request::i()->id );
			$page->enableToggle($page->open);
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_page_not_found', '2BIMVB101/3', 404 );
		}

		/* Redirect */
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=pages' ), 'saved' );
	}	
}