<?php


namespace IPS\videobox\modules\admin\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	
	public function execute()
	{
		if ( \IPS\Request::i()->do != 'player' )
		{
			\IPS\Output::i()->sidebar['actions'] = array();

			\IPS\Output::i()->sidebar['actions']['player'] = array(
				'icon'	=> 'youtube-play',
				'link'	=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=settings&do=player' ),
				'title'	=> 'videobox_player',
			);	
			
			\IPS\Output::i()->sidebar['actions']['mediatags'] = array(
				'icon'	=> 'code',
				'link'	=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags' ),
				'title'	=> 'videobox_metatags',
			);	
		}
		
		\IPS\Dispatcher::i()->checkAcpPermission( 'servers_manage' );
		
		parent::execute();
	}
	
	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		# button on top
		\IPS\Output::i()->sidebar['actions'] = array();

		\IPS\Output::i()->sidebar['actions']['player'] = array(
			'icon'	=> 'youtube-play',
			'link'	=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=settings&do=player' ),
			'title'	=> 'videobox_player',
		);	
		
		\IPS\Output::i()->sidebar['actions']['mediatags'] = array(
			'icon'	=> 'code',
			'link'	=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=mediatags' ),
			'title'	=> 'videobox_metatags',
		);	
		
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );
		
		$form = new \IPS\Helpers\Form;	

		/* What's new block */
		$form->addTab('vb_conf_index');		
		$form->addHeader('vb_conf_home');
		$blocksOptions = array( 
			'trending'		=> 'vb_home_block_trending',
			'new' 			=> 'vb_home_block_new',
			//'followed' 	=> 'vb_home_block_followed',
			'featured' 		=> 'vb_home_block_featured',
		);
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vb_conf_index_new_on', explode(',', \IPS\Settings::i()->vb_conf_index_new_on), FALSE, array( 'options' => $blocksOptions ) ) );
		$blocksStyles = array( 
			'blocks'		=> 'vb_home_style_blocks',
			'tabs' 			=> 'vb_home_style_tabs'
		);
		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_index_new_style', \IPS\Settings::i()->vb_conf_index_new_style, FALSE, array( 'options' => $blocksStyles ) ) );		
		$form->add( new \IPS\Helpers\Form\Number( 'vb_conf_index_new_items', \IPS\Settings::i()->vb_conf_index_new_items, FALSE, array(), NULL, NULL, NULL, 'vb_conf_index_new_items' ) );
		$form->add( new \IPS\Helpers\Form\Node( 'vb_conf_index_new_cat', ( \IPS\Settings::i()->vb_conf_index_new_cat AND \IPS\Settings::i()->vb_conf_index_new_cat != 0 ) ? explode( ',', \IPS\Settings::i()->vb_conf_index_new_cat ) : 0, FALSE, array(
			'class'           => '\IPS\videobox\Category',
			'multiple'        => true,
			'clubs' 		  => true, 
			'zeroVal' 		  => 'any'
		), NULL, NULL, NULL, 'vb_conf_index_new_cat' ) );	
		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_index_new_order', \IPS\Settings::i()->vb_conf_index_new_order ?: 'updated', FALSE, array( 'options' => array( 
			'updated' 		=> 'sort_updated', 
			'last_comment' 	=> 'last_reply', 
			'title' 		=> 'videobox_title', 
			'rating' 		=> 'sort_rating', 
			'date' 			=> 'sort_date', 
			'num_comments' 	=> 'sort_num_comments', 
			'num_reviews' 	=> 'sort_num_reviews', 
			'views' 		=> 'sort_num_views', 
			'online' 		=> 'sort_online' 
		) ), NULL, NULL, NULL, 'vb_conf_index_new_order' ) );		
		
		/* Other */
		$form->addHeader('vb_conf_other');		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_other_postbutton', \IPS\Settings::i()->vb_conf_other_postbutton, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_rss', \IPS\Settings::i()->vb_conf_rss, FALSE, array() ) );		
		
		/* Images */
		$form->addTab('vb_conf_images');					
		$form->add( new \IPS\Helpers\Form\WidthHeight( 'vb_conf_thumbnail_medium', \IPS\Settings::i()->vb_conf_thumbnail_medium ? explode( ':', \IPS\Settings::i()->vb_conf_thumbnail_medium ) : array( 320, 180 ), FALSE, array('resizableDiv' => FALSE), NULL, NULL, NULL, 'vb_conf_thumbnail_medium' ) );		
		$form->add( new \IPS\Helpers\Form\WidthHeight( 'vb_conf_featured_size', \IPS\Settings::i()->vb_conf_featured_size ? explode( ':', \IPS\Settings::i()->vb_conf_featured_size ) : array( 1277, 498 ), FALSE, array('resizableDiv' => FALSE) ) );
		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_featured_whocanupload', \IPS\Settings::i()->vb_conf_featured_whocanupload ? array_filter( explode( ',', \IPS\Settings::i()->vb_conf_featured_whocanupload ) ) : FALSE, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ), NULL, NULL, NULL, 'vb_conf_featured_whocanupload' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'vb_conf_thumbnail_uploadsize', \IPS\Settings::i()->vb_conf_thumbnail_uploadsize ?  \IPS\Settings::i()->vb_conf_thumbnail_uploadsize : 2, FALSE, array(), NULL, NULL, NULL, 'vb_conf_thumbnail_uploadsize' ) );
		$form->add( new \IPS\Helpers\Form\Custom( 'vb_conf_imgrebuild', NULL, FALSE, array( 'getHtml' => function( $element )
		{		
			return \IPS\Member::loggedIn()->language()->addToStack('vb_conf_imgrebuild_link');
		} ) ) );				

		/* Video view */
		$form->addTab('vb_conf_online');
		$form->addHeader('vb_conf_online_basic');		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_online_on', \IPS\Settings::i()->vb_conf_online_on, FALSE, array( 'togglesOn' => array( 'vb_conf_online_cats', 'vb_conf_videouploadsize', 'vb_conf_online_groups', 'vb_conf_online_post' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Node( 'vb_conf_online_cats', ( \IPS\Settings::i()->vb_conf_online_cats and \IPS\Settings::i()->vb_conf_online_cats != '*' ) ? \IPS\Settings::i()->vb_conf_online_cats : 0, FALSE, array(
			'class'           => '\IPS\videobox\Category',
			'zeroVal'         => 'all',
			'permissionCheck' => 'view',
			'multiple'        => true
		), NULL, NULL, NULL, 'vb_conf_online_cats' ) );	
		
		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_online_groups', \IPS\Settings::i()->vb_conf_online_groups ? array_filter( explode( ',', \IPS\Settings::i()->vb_conf_online_groups ) ) : FALSE, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ), NULL, NULL, NULL, 'vb_conf_online_groups' ) );
		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_online_post', \IPS\Settings::i()->vb_conf_online_post ? array_filter( explode( ',', \IPS\Settings::i()->vb_conf_online_post ) ) : FALSE, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ), NULL, NULL, NULL, 'vb_conf_online_post' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'vb_conf_videouploadsize', \IPS\Settings::i()->vb_conf_videouploadsize ?  \IPS\Settings::i()->vb_conf_videouploadsize : 10, FALSE, array(), NULL, NULL, NULL, 'vb_conf_videouploadsize' ) );

		/* Live Stream */
		$form->addTab('vb_conf_lv_tab');
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_lv_on', \IPS\Settings::i()->vb_conf_lv_on, FALSE, array() ) );
		
		// $form->add( new \IPS\Helpers\Form\Node( 'vb_conf_lv_cat', \IPS\Settings::i()->vb_conf_lv_cat ? \IPS\Settings::i()->vb_conf_lv_cat : 0, FALSE, array(
			// 'class'           => '\IPS\videobox\Category',
			// 'permissionCheck' => 'view',
			// 'multiple'        => true
		// ), NULL, NULL, NULL, 'vb_conf_lv_cat' ) );			
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_lv_chat', \IPS\Settings::i()->vb_conf_lv_chat, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_lv_allow_thumb', \IPS\Settings::i()->vb_conf_lv_allow_thumb, FALSE, array() ) );
		
		$typeOptions = array( 	'twitch' 		=> 'vb_lv_twitch_on', 
								'mixer' 		=> 'vb_lv_mixer_on',
								'smashcast' 	=> 'vb_lv_smashcast_on',
								'youtube' 		=> 'vb_lv_youtube_on' 
		);
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vb_conf_lv_services', array_filter( explode( ',', \IPS\Settings::i()->vb_conf_lv_services ) ), FALSE, array( 'options' => $typeOptions ) ) );
			
		/* POINTS */
        $form->addTab( 'vb_conf_points_tab' );
        $form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_points_on', \IPS\Settings::i()->vb_conf_points_on, FALSE, array( 'togglesOn' => array( 'vb_conf_points_table', 'vb_conf_points_column', 'vb_conf_points_video', 'vb_conf_points_comment', 'vb_conf_points_review', 'vb_conf_points_reaction', 'vb_conf_points_decrease' ) ) ) );
        $form->add( new \IPS\Helpers\Form\Text( 'vb_conf_points_table', \IPS\Settings::i()->vb_conf_points_table, FALSE, array(), NULL, NULL, NULL, 'vb_conf_points_table' ) );
        $form->add( new \IPS\Helpers\Form\Text( 'vb_conf_points_column', \IPS\Settings::i()->vb_conf_points_column, FALSE, array(),  NULL, NULL, NULL, 'vb_conf_points_column' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vb_conf_points_video', \IPS\Settings::i()->vb_conf_points_video, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vb_conf_points_video' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vb_conf_points_comment', \IPS\Settings::i()->vb_conf_points_comment, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vb_conf_points_comment' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vb_conf_points_review', \IPS\Settings::i()->vb_conf_points_review, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vb_conf_points_review' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vb_conf_points_reaction', \IPS\Settings::i()->vb_conf_points_reaction, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vb_conf_points_reaction' ) );
        $form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_points_decrease', \IPS\Settings::i()->vb_conf_points_decrease, FALSE, array(), NULL, NULL, NULL, 'vb_conf_points_decrease' ) );
		
		/* API config */
		$form->addTab('vb_conf_api_tab');
		$form->add( new \IPS\Helpers\Form\Text( 'vb_conf_lv_twitch_api', \IPS\Settings::i()->vb_conf_lv_twitch_api, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Text( 'vb_conf_api_youtube', \IPS\Settings::i()->vb_conf_api_youtube, FALSE, array() ) );
		
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			$values['vb_conf_featured_size'] = $values['vb_conf_featured_size'] ? implode( ':', explode(",",$values['vb_conf_featured_size']) ) : NULL;			
			$values['vb_conf_thumbnail_medium'] = $values['vb_conf_thumbnail_medium'] ? implode( ':', explode(",",$values['vb_conf_thumbnail_medium']) ) : NULL;
			$form->saveAsSettings( $values );
			\IPS\Session::i()->log( 'acplogs__videobox_settings' );
		}
		
		/* Output */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('settings');
		\IPS\Output::i()->output = $form;
	}
	
	/**
	 * Player settings
	 */	
	protected function player()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );
		
		$form = new \IPS\Helpers\Form;
		$form->addHeader('videobox_manaSub');
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_online_sub', \IPS\Settings::i()->vb_conf_online_sub, FALSE, array(), NULL, NULL, NULL, 'vb_conf_online_sub' ) );		
		
		$form->addHeader('vb_conf_online_watermark_header');
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_online_watermark_on', \IPS\Settings::i()->vb_conf_online_watermark_on, FALSE, array( 'togglesOn' => array( 'vb_conf_online_watermark' ) ) ) );		
		$form->add( new \IPS\Helpers\Form\Upload( 'vb_conf_online_watermark', \IPS\Settings::i()->vb_conf_online_watermark ? \IPS\File::get( 'core_Theme', \IPS\Settings::i()->vb_conf_online_watermark ) : NULL, FALSE, array( 'image' => true, 'storageExtension' => 'core_Theme' ), NULL, NULL, NULL, 'vb_conf_online_watermark' ) );

		$form->addHeader('vb_conf_online_ga_header');
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_online_ga_on', \IPS\Settings::i()->vb_conf_online_ga_on, FALSE, array( 'togglesOn' => array( 'vb_conf_online_ga' ) ) ) );		
		
		$form->addHeader('vb_conf_online_tab_preroll');
		$form->add( new \IPS\Helpers\Form\YesNo( 'vb_conf_online_preroll', \IPS\Settings::i()->vb_conf_online_preroll, FALSE, array( 'togglesOn' => array( 'vb_conf_online_preroll_src', 'vb_conf_online_preroll_href', 'vb_conf_online_preroll_skip' ) ) ) );		
		$form->add( new \IPS\Helpers\Form\Upload( 'vb_conf_online_preroll_src', \IPS\Settings::i()->vb_conf_online_preroll_src ? \IPS\File::get( 'core_Theme', \IPS\Settings::i()->vb_conf_online_preroll_src ) : NULL, FALSE, array( 'allowedFileTypes' => array( 'mp4' ), 'storageExtension' => 'core_Theme' ), NULL, NULL, NULL, 'vb_conf_online_preroll_src' ) );
		$form->add( new \IPS\Helpers\Form\Url( 'vb_conf_online_preroll_href', \IPS\Settings::i()->vb_conf_online_preroll_href, FALSE, array(), NULL, NULL, NULL, 'vb_conf_online_preroll_href' ) );	
		$form->add( new \IPS\Helpers\Form\Number( 'vb_conf_online_preroll_skip', \IPS\Settings::i()->vb_conf_online_preroll_skip, FALSE, array(), NULL, NULL, NULL, 'vb_conf_online_preroll_skip' ) );	
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			$form->saveAsSettings( $values );
		}
		
		/* Output */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('settings');
		\IPS\Output::i()->output = $form;
	}	
	
	/**
	 * Rebuild Thumbnal
	 */
	protected function rebuildThumbnails()
	{
		$perGo = 10;
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('videobox_rebuildimages');
		
		if( \IPS\Db::i()->checkForColumn( 'videobox_videos', 'v_thumbnail_small' ) )
		{
			\IPS\Output::i()->output = new \IPS\Helpers\MultipleRedirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=settings&do=rebuildThumbnails' ), 
			function( $done ) use( $perGo )
			{
				$select = \IPS\Db::i()->select( '*', 'videobox_videos', "v_thumbnail_small IS NOT NULL AND v_thumbnail_small != ''", 'v_id', array( $done, $perGo ), NULL, NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS );
				$total	= $select->count( TRUE );
				
				if ( !$select->count() )
				{
					return NULL;
				}
							
				foreach ( $select as $row )
				{
					try
					{
						\IPS\File::get( 'videobox_IMG', $row['v_thumbnail_small'] )->delete();				
					}
					catch ( \Exception $e ) {}
				}
				
				$done += $perGo;		
				return array( $done, \IPS\Member::loggedIn()->language()->addToStack( 'vb_rebuildingThumb' , FALSE, array( 'sprintf' => $done < $total ? $done : $total ) ), ( 100 * $done ) / $total );
			}, function()
			{
				\IPS\Db::i()->dropColumn( 'videobox_videos', array( 'v_thumbnail_small' ) );
				\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=settings&do=rebuildThumbnails' ), 'completed' );
			} );			
		}
		else
		{
			\IPS\Output::i()->output = new \IPS\Helpers\MultipleRedirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=settings&do=rebuildThumbnails' ), 
			function( $doneSoFar ) use( $perGo )
			{
				$select = \IPS\Db::i()->select( '*', 'videobox_videos', "v_thumbnail IS NOT NULL AND v_thumbnail != ''", 'v_id', array( $doneSoFar, $perGo ), NULL, NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS );
				$total	= $select->count( TRUE );
				
				if ( !$select->count() )
				{
					return NULL;
				}
							
				foreach ( $select as $row )
				{
					try
					{
						$original = \IPS\File::get( 'videobox_IMG', $row['v_thumbnail'] );
						
						\IPS\File::get( 'videobox_IMG', $row['v_thumbnail_medium'] )->delete();
						
						$medium = \IPS\Application::load('videobox')->createThumb( \IPS\Settings::i()->vb_conf_thumbnail_medium, $original, $row['v_thumbnail_medium'], 'videobox_IMG' );
						
						\IPS\Db::i()->update( 'videobox_videos', array( 'v_thumbnail_medium' => $medium ), array( 'v_id=?', $row['v_id'] ) );	
						
						if ( $row['v_featuredIMG'] )
						{
							$fimg = \IPS\File::get( 'videobox_Slider', $row['v_featuredIMG_original'] );	
							
							\IPS\File::get( 'videobox_Slider', $row['v_featuredIMG'] )->delete();
							
							$featuredIMG = \IPS\Application::load('videobox')->createThumb( \IPS\Settings::i()->vb_conf_featured_size, $fimg, $row['v_featuredIMG'], 'videobox_Slider' );
							
							\IPS\Db::i()->update( 'videobox_videos', array( 'v_featuredIMG'	=> $featuredIMG ), array( 'v_id=?', $row['v_id'] ) );							
						}					
					}
					catch ( \Exception $e ) {}
				}
				
				$doneSoFar += $perGo;		
				return array( $doneSoFar, \IPS\Member::loggedIn()->language()->addToStack( 'vb_rebuildingThumb' , FALSE, array( 'sprintf' => $doneSoFar < $total ? $doneSoFar : $total ) ), ( 100 * $doneSoFar ) / $total );
			}, function()
			{
				\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=settings' ), 'completed' );
			} );
		}
	}	
}