<?php


namespace IPS\videobox\modules\front\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _browse extends \IPS\Dispatcher\Controller
{
	/**
	 * Route
	 *
	 * @return	void
	 */
	protected function manage()
	{
		if ( isset( \IPS\Request::i()->id ) )
		{
			try
			{
				$this->_category( \IPS\videobox\Category::loadAndCheckPerms( \IPS\Request::i()->id, 'read' ) );
			}
			catch ( \OutOfRangeException $e )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMVB103/1', 404 );
			}
		}
		elseif ( \IPS\Request::i()->do == 'whatnew' )
		{
			$this->whatnew();
		}
		else
		{
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index', 'front', 'videobox' ) );
		}		
	}
	
	/**
	 * Show Category
	 *
	 * @param	\IPS\videobox\Category	$category	The category to show
	 * @return	void
	 */
	protected function _category( $category )
	{
		$catURL = $category->url()->setQueryString( \IPS\Request::i()->url()->queryString );

		$where = array();
		
		# Alphabet
		if ( $category->bitoptions['showAlphabet'] )
		{
			$alphabet = isset( \IPS\Request::i()->alphabet ) ? \IPS\Request::i()->alphabet : 'all';
			
			if ( isset( $alphabet ) )
			{		
				if ( mb_strtolower( $alphabet ) != 'all' )
				{
					$where[] = array( "videobox_videos.v_title LIKE '" . \IPS\Db::i()->real_escape_string( mb_strtolower( $alphabet ) ) . "%'" );
				}				
			}
		}
		
		# Tags Filter
		if ( \IPS\Settings::i()->tags_enabled && $category->tags_filter_on == 1 && \IPS\Request::i()->tagsfilter )
		{
			foreach( explode( '|', \IPS\Request::i()->tagsfilter ) as $tag_text )
			{
				$tagConds[] = "t.tag_cache_text LIKE '%{$tag_text}%'";
			}
			$where[] = array( implode( " AND ", $tagConds ) );
		}
		
		# Build table
		if ( $category->hasChildren() )
		{
			$catIDs = \IPS\videobox\Category::getChildCats( $category );
			
			$where[] = array( "videobox_videos.v_cat IN (" . implode(",", $catIDs) . ")" );

			$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $catURL, $where );
		}
		else
		{
			$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $catURL, $where, $category );
		}
		
		$table->sortOptions = array_merge( $table->sortOptions, array( 'online' => 'v_livestream_online' ) );		
		$table->sortBy = \IPS\Request::i()->sortby ? \IPS\Request::i()->sortby : $category->sortorder;
		$table->sortDirection = mb_strpos( $table->sortBy, "title" ) !== false ? 'ASC' : 'DESC';

		$table->limit = $category->videoPerPage > 0 ? $category->videoPerPage : 30;	
		$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), \IPS\Application::load('videobox')->getTableRowsTemplate() );
		
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and $category->nexus_on )
		{
			$table->filters = array(
				'video_free'	=> "( v_cost='' OR v_cost IS NULL )",
				'video_paid'	=> "( v_cost<>'' OR v_cost IS NOT NULL )",
			);
		}
			
		/* Online User Location */
		$permissions = $category->permissions();
		\IPS\Session::i()->setLocation( $category->url(), explode( ",", $permissions['perm_view'] ), 'loc_videobox_viewing_category', array( "videobox_category_{$category->id}" => TRUE ) );
						
		/* Output */
		\IPS\Output::i()->title	= $category->_title;
		
		\IPS\Output::i()->contextualSearchOptions[ \IPS\Member::loggedIn()->language()->addToStack( 'search_contextual_item', FALSE, array( 'sprintf' => array( \IPS\Member::loggedIn()->language()->addToStack( $category->_title ) ) ) ) ] = array( 'type' => 'videobox_video', 'nodes' => $category->_id );		
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->category( $category, (string) $table );
	}
	
	protected function add()
	{
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=post&do=post&id=' . \IPS\Request::i()->id ) );
	}
	
	protected function whatnew()
	{
		$n_where = array();
		if ( \IPS\Settings::i()->vb_conf_index_new_ignoreCat )
		{
			$n_ignoreCat = explode( ",", \IPS\Settings::i()->vb_conf_index_new_ignoreCat );
			$n_where[] = array( 'v_cat NOT IN (' . implode( ",", $n_ignoreCat ) . ')' );
		}			
		$url = \IPS\Http\Url::internal( 'app=videobox', 'front', 'videobox' );
		$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $url, $n_where );
		$table->limit = \IPS\Settings::i()->vb_conf_index_new_items > 0 ? \IPS\Settings::i()->vb_conf_index_new_items : 30;
		$table->title = \IPS\Member::loggedIn()->language()->addToStack('videobox_whats_new');
		$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), \IPS\Application::load('videobox')->getTableRowsTemplate() );

		$table->sortOptions = array_merge( $table->sortOptions, array( 'online' => 'v_livestream_online' ) );		
		$table->sortBy = \IPS\Request::i()->sortby ? \IPS\Request::i()->sortby : \IPS\Settings::i()->vb_conf_index_new_order;
		$table->sortDirection = $table->sortBy == 'title' || $table->sortBy == 'v_title' || $table->sortBy == 'videobox_videos.v_title' ? 'ASC' : 'DESC';
		
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\Settings::i()->vb_conf_nexus_on )
		{
			$table->filters = array(
				'video_free'	=> "( v_cost='' OR v_cost IS NULL )",
				'video_paid'	=> "( v_cost<>'' OR v_cost IS NOT NULL )",
			);
		}
		
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack('videobox_whats_new');
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack('videobox_whats_new') );	
		\IPS\Output::i()->output = (string) $table;
	}
	
}