<?php


namespace IPS\videobox\modules\front\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{	
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{	
		/* Add RSS feed */
		if ( \IPS\Settings::i()->vb_conf_rss )
		{
			\IPS\Output::i()->rssFeeds['vb_rss_title'] = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index&do=rss', 'front', 'videobox_rss' );
		}
			
		/* Page Title */
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack('__app_videobox');
		
		/* Data */
		$tabs = explode(',', \IPS\Settings::i()->vb_conf_index_new_on);
		
		if ( \IPS\Settings::i()->vb_conf_index_new_style == 'tabs' )
		{
			$area = \IPS\Request::i()->area ? \IPS\Request::i()->area : $tabs[0];
			
			if ( \in_array( $area, array( 'trending', 'new', 'featured' ) ) && \in_array( $area, $tabs ) )
			{
				$url = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index&area=' . $area, 'front', 'videobox_tab_' . $area );
				
				$table = $this->getBlock( $area, $url );
				
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->sendOutput( (string) $table );
				}
				else
				{
					\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->indexTabs( $table, $url, $area );
				}
			}
			else
			{
				$url = \IPS\Http\Url::internal( 'app=videobox', 'front', 'videobox' );
			}
		}
		else
		{
			$url = \IPS\Http\Url::internal( 'app=videobox', 'front', 'videobox' );
			
			if ( \in_array( 'trending', $tabs ) )
			{
				$urlTrending = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index&area=trending', 'front', 'videobox_tab_trending' );
				$trendingBlock = $this->getBlock( 'trending', $urlTrending );
			}
			if ( \in_array( 'new', $tabs ) )
			{			
				$urlNew = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index&area=new', 'front', 'videobox_tab_new' );
				$newBlock = $this->getBlock( 'new', $urlNew );
			}
			if ( \in_array( 'featured', $tabs ) )
			{			
				$urlFeatured = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index&area=featured', 'front', 'videobox_tab_featured' );
				$featuredBlock = $this->getBlock( 'featured', $urlFeatured );
			}
			
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->indexBlocks( $trendingBlock, $newBlock, $featuredBlock );
		}
		
		/* Online User Location */
		\IPS\Session::i()->setLocation( $url, array(), 'loc_videobox_browsing' );		
	}
	
	/**
	 * get Block
	 *
	 * @return	table data
	 */	
	protected function getBlock($area, $url)
	{
		$where = array();
		if ( \IPS\Settings::i()->vb_conf_index_new_cat && \IPS\Settings::i()->vb_conf_index_new_cat != 0 )
		{
			$where[] = array( 'v_cat IN (' . \IPS\Settings::i()->vb_conf_index_new_cat . ')' );
		}	

		if ( $area == 'trending' )
		{
			// Reset Time
			$canReset = FALSE;
			if ( date('N', time()) == 1 && time() - \IPS\Settings::i()->vb_trending_lastReset >= 604800) // Reset vao thu hai moi tuan
			{
				$canReset = TRUE;	
			}
			if ( $canReset )
			{
				\IPS\Db::i()->update( 'videobox_videos', "`v_trending`=0", array( "v_trending>?", 0 ) );
				\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => time() ), array( 'conf_key=?', 'vb_trending_lastReset' ) );
				\IPS\Settings::i()->vb_trending_lastReset = time();
				unset( \IPS\Data\Store::i()->settings );						
			}
			
			//$where[] = array( 'v_trending>?', 0 );
			$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $url, $where );	
			$table->sortOptions = array_merge( $table->sortOptions, array( 'trending' => 'v_trending' ) );
			$table->sortBy = 'v_trending';
			$table->sortDirection = 'DESC';
		}
		elseif ( $area == 'featured' )
		{
			$where[] = array( 'v_featured=?', 1 );
			$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $url, $where );
			$table->sortOptions = array_merge( $table->sortOptions, array( 'online' => 'v_livestream_online' ) );		
			$table->sortBy = \IPS\Request::i()->sortby ?: \IPS\Settings::i()->vb_conf_index_new_order;
			$table->sortDirection = 'DESC';
		}
		else
		{
			$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $url, $where );
			$table->sortOptions = array_merge( $table->sortOptions, array( 'online' => 'v_livestream_online' ) );		
			$table->sortBy = \IPS\Request::i()->sortby ?: \IPS\Settings::i()->vb_conf_index_new_order;
			$table->sortDirection = 'DESC';			
		}
		
		if ( \IPS\Settings::i()->vb_conf_index_new_style == 'blocks' )
		{
			$table->title = \IPS\Member::loggedIn()->language()->addToStack( 'vbTabs_' . $area );
		}
		
		$table->limit = \IPS\Settings::i()->vb_conf_index_new_items > 0 ? \IPS\Settings::i()->vb_conf_index_new_items : 20;
		$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), \IPS\Application::load('videobox')->getTableRowsTemplate() );			
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\videobox\Category::hasPaidCategory() )
		{
			$table->filters = array(
				'video_free'	=> "( v_cost='' OR v_cost IS NULL )",
				'video_paid'	=> "( v_cost<>'' OR v_cost IS NOT NULL )",
			);
		}			

		return $table;
	}
	
	/**
	 * Latest Videos RSS
	 *
	 * @return	void
	 */
	protected function rss()
	{
		if( !\IPS\Settings::i()->vb_conf_rss )
		{
			\IPS\Output::i()->error( 'rss_offline', '2BIMVB105/1', 404, 'rss_offline_admin' );
		}

		$document = \IPS\Xml\Rss::newDocument( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index', 'front', 'videobox' ), \IPS\Member::loggedIn()->language()->get('vb_rss_title'), \IPS\Member::loggedIn()->language()->get('vb_rss_title') );
		
		foreach ( \IPS\videobox\Video::getItemsWithPermission() as $video )
		{
			$thumbnail = "<img src=\"{$video->thumbnail('medium')}\" style='align: center; margin: 0 auto;'><br>";
			$document->addItem( $video->title, $video->url(), $thumbnail . $video->content, \IPS\DateTime::ts( $video->updated ), $video->id );
		}
		
		/* @note application/rss+xml is not a registered IANA mime-type so we need to stick with text/xml for RSS */
		\IPS\Output::i()->sendOutput( $document->asXML(), 200, 'text/xml' );
	}	
}