<?php


namespace IPS\videobox\modules\front\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * post
 */
class _post extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$form = new \IPS\Helpers\Form( 'videobox_select_category', 'continue' );
		$form->class = 'ipsForm_vertical ipsForm_noLabels';
		$form->add( new \IPS\Helpers\Form\Node( 'category', isset( $data['category'] ) ? $data['category'] : NULL, TRUE, array(
			'url'					=> \IPS\Http\Url::internal( 'app=videobox&module=video&controller=post', 'front', 'videobox_post' ),
			'class'					=> 'IPS\videobox\Category',
			'permissionCheck'		=> function( $node )
			{
				if ( $node->can( 'view' ) )
				{
					if ( $node->can( 'add' ) )
					{
						return TRUE;
					}
					
					return FALSE;
				}
				
				return NULL;
			},
		) ) );

		if ( $values = $form->values() )
		{
			$url = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=post&do=post', 'front', 'videobox_post' )->setQueryString( 'id', $values['category']->_id );
			\IPS\Output::i()->redirect( $url );
		}
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'videobox_select_category' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'post' )->categorySelector( $form );
	}
	
	protected function post()
	{
		/* Limit posting videos */
		if ( \IPS\Member::loggedIn()->group['videobox_limit_videos'] > 0 )
		{
			$totalVideos = \IPS\Db::i()->select( 'COUNT(*)', 'videobox_videos', array( "v_author=?", \IPS\Member::loggedIn()->member_id ) )->first();
			
			if ( $totalVideos >= \IPS\Member::loggedIn()->group['videobox_limit_videos'] )
			{
				\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('vb_error_limit_posting', FALSE, array( 'sprintf' => \IPS\Member::loggedIn()->group['videobox_limit_videos'] ) ), '2BIMVB107/1', 403 );
			}
		}
		
		/* JS */
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'front_video.js', 'videobox', 'front' ) );
		
		/* Get category data */
		$category = NULL;
		
		if ( isset( \IPS\Request::i()->id ) )
		{
			try
			{
				$category = \IPS\videobox\Category::loadAndCheckPerms( \IPS\Request::i()->id );
			}
			catch ( \OutOfRangeException $e ) { }
		}
		
		if ( !$category )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMVB107/2', 403 );
		}		
		
		/* Form */
		$form = \IPS\videobox\Video::create( $category );
		$form->hiddenValues['category'] = $category->_id;

		$formTemplate = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'post', 'videobox' ) ), 'postForm' ), $category, TRUE );

		/* Display */
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'videobox_post' );
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title		= $title;
		\IPS\Output::i()->breadcrumb[] = array( $category->url(), $category->_title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'post' )->postPage( $category, $formTemplate, $title );
	}

}