<?php


namespace IPS\videobox\modules\front\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * quicksearch
 */
class _quicksearch extends \IPS\Dispatcher\Controller
{

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		# Load QuickSearch
		$termID = \IPS\Request::i()->id ? (int) \IPS\Request::i()->id : 0;
		
		try
		{
			$term = \IPS\videobox\QuickSearch\Term::load( $termID );	
		}
		catch ( \OutOfRangeException $e ) {}
		
		if ( !$term )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMVB108/1', 403, '' );
		}
		
		$group = $term->parent();
		
		if ( !$group->canUse() )
		{
			\IPS\Output::i()->error( 'videobox_nopermission', '2BIMVB108/2', 403, '' );
		}		

		# Load Field
		try
		{
			$field = \IPS\videobox\Field::load( $group->field );
		}
		catch ( \OutOfRangeException $e ) {}

		if ( $field )
		{		
			# Search term
			$searchterm = mb_strtolower( $term->value );

			if ( $field->type == 'CheckboxSet' )
			{
				foreach( json_decode($field->content) as $k => $val )
				{
					if ( mb_strtolower($val) == $searchterm )
					{
						$query = "find_in_set('{$k}', field.field_{$group->field})";
						break;
					}
				}
			}
			else
			{
				switch ( $group->mode )
				{
					case 'like':
						$query = array( "field.field_{$group->field} LIKE '%?%'", $searchterm );
						break;				
					case 'equal':
						$query = array( "field.field_{$group->field}=?", $searchterm );
						break;
					case 'greater':
						$query = array( "field.field_{$group->field}>=?", $searchterm );
						break;
					case 'lessthan':
						$query = array( "field.field_{$group->field}<=?", $searchterm );					
						break;
					case 'different':
						$query = array( "field.field_{$group->field}!=?", $searchterm );		
						break;			
				}	
			}
		}
		
		if ( $group->cat != '*' )
		{
			$where[] = array( "v_cat IN (" . $group->cat . ")" );
		}
		
		$alphabet = isset(\IPS\Request::i()->alphabet) ? \IPS\Request::i()->alphabet : 'all';
		
		$sURL = isset($alphabet) ? $term->url()->setQueryString( array( 'alphabet' => $alphabet ) ) : $term->url();
		
		if ( isset($alphabet) && mb_strtolower( $alphabet ) != 'all' )
		{
			$where[] = array( "videobox_videos.v_title LIKE '" . \IPS\Db::i()->real_escape_string( mb_strtolower( $alphabet ) ) . "%'" );
		}
		
		if ( $query )
		{
			$where[] = $query;
		}
		
		# Load table
		$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $sURL, $where );
		$table->limit = 24;	
		$table->title = $term->_title;
		$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), \IPS\Application::load('videobox')->getTableRowsTemplate() );
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\videobox\Category::hasPaidCategory() )
		{
			$table->filters = array(
				'video_free'	=> "( v_cost='' OR v_cost IS NULL )",
				'video_paid'	=> "( v_cost<>'' OR v_cost IS NOT NULL )",
			);
		}
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( $term->url(), $term->_title );		
		\IPS\Output::i()->title		= $term->_title;	
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->searchResult( $term->url(), $table );
	}
	
	
}