<?php

namespace IPS\videobox\modules\front\video;

/* To prvideo PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * View Video Controller
 */
class _view extends \IPS\Content\Controller
{
	/**
	 * [Content\Controller]	Class
	 */
	protected static $contentModel = 'IPS\videobox\Video';

	/**
	 * Init
	 *
	 * @return	void
	 */
	public function execute()
	{		
		try
		{
			$this->video = \IPS\videobox\Video::load( \IPS\Request::i()->id );

			if ( !$this->video->canView( \IPS\Member::loggedIn() ) )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMVB110/1', 403 );
			}
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMVB110/2', 404 );
		}

		parent::execute();
	}
	
	/**
	 * View Video
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Init */
		parent::manage();

		/* AJAX hover preview? */
		if ( \IPS\Request::i()->preview )
		{
			\IPS\Output::i()->sendOutput( \IPS\Theme::i()->getTemplate( 'view' )->videoPreview($this->video, $likes) );
			return;
		}	
		/* Update counter for trending */
		\IPS\Db::i()->update( 'videobox_videos', "`v_trending`=`v_trending`+1", array( "v_id=?", $this->video->id ) );		
		
		/* Work out output */
		$area = \IPS\Request::i()->area ? \IPS\Request::i()->area : 'info';
		$output = $area == 'online' ? $this->_online() : $this->_info();
		
		/* Sort out comments and reviews */		
		$tabs = $this->video->commentReviewTabs();
		$_tabs = array_keys( $video );
		$tab = isset( \IPS\Request::i()->tab ) ? \IPS\Request::i()->tab : array_shift( $_tabs );
		$activeTabContents = $this->video->commentReviews( $tab );
		$commentsAndReviews = \count( $tabs ) ? \IPS\Theme::i()->getTemplate( 'global', 'core' )->commentsAndReviewsTabs( \IPS\Theme::i()->getTemplate( 'global', 'core' )->tabs( $tabs, $tab, $activeTabContents, $this->video->url(), 'tab', FALSE, TRUE ), md5( $this->video->url() ) ) : NULL;
	
		if ( \IPS\Request::i()->isAjax() && !isset( \IPS\Request::i()->rating_submitted ) && !isset( \IPS\Request::i()->lightbox ) )
		{
			\IPS\Output::i()->output = $activeTabContents;
			return;
		}		

		/* Online User Location */
		\IPS\Session::i()->setLocation( $this->video->url(), $this->video->onlineListPermissions(), 'loc_videobox_viewing_video', array( $this->video->title => FALSE ) );
		
		/* Meta tags */
		\IPS\Output::i()->metaTags['og:image'] = $this->video->thumbnail();
		\IPS\Output::i()->metaTags['og:type'] = \IPS\Member::loggedIn()->language()->addToStack('videobox_meta_ogtype');
		
		/* Search */
		\IPS\Output::i()->contextualSearchOptions = array();
		\IPS\Output::i()->contextualSearchOptions[ \IPS\Member::loggedIn()->language()->addToStack( 'videobox_category_' . $this->video->container()->id ) ] = array( 'type' => 'videobox_video', 'nodes' => $this->video->container()->id );

		/* JS */
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'front_video.js', 'videobox', 'front' ) );
		
		/* Output */			
		\IPS\Output::i()->title = $this->video->title;
		
		$html = \IPS\Theme::i()->getTemplate( 'view' )->view( $area, $commentsAndReviews, $output, $this->video );
		
		\IPS\Output::i()->output = $html;	
	}

	//-----------------------------
	// Change Author
	//-----------------------------	
	public function changeAuthor()
	{
		/* Permission check */
		if ( !$this->video->canChangeAuthor() )
		{
			\IPS\Output::i()->error( 'no_module_permission', '2BIMVB110/3', 403 );
		}
		
		/* Build form */
		$form = new \IPS\Helpers\Form;
		$form->add( new \IPS\Helpers\Form\Member( 'author', NULL, TRUE ) );
		$form->class .= 'ipsForm_vertical';

		/* Handle submissions */
		if ( $values = $form->values() )
		{
			$this->video->changeAuthor( $values['author'] );			
			\IPS\Output::i()->redirect( $this->video->url() );
		}
		
		/* Display form */
		\IPS\Output::i()->output = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );;
	}
	
	//-----------------------------
	// Video Info
	//-----------------------------
	protected function _info()
	{		
		if ( \IPS\Settings::i()->vb_conf_online_on == 1 && $this->video->container()->videoTab != 1 && ( \IPS\Settings::i()->vb_conf_online_cats=='0' || \in_array( $this->video->container()->id, explode(",",\IPS\Settings::i()->vb_conf_online_cats) ) ) )
		{
			$this->video->videoJS();
		}
		return \IPS\Theme::i()->getTemplate( 'view' )->videoInfo( $this->video );
	}

	//-----------------------------
	// Watch Video Online
	//-----------------------------
	protected function _online()
	{	
		/* Can we play? */
		try
		{
			$this->video->playCheck();
		}
		catch ( \DomainException $e )
		{
			\IPS\Output::i()->error( $e->getMessage(), '2BIMVB110/4', 403, '' );
		}	
		
		$this->video->videoJS();
		return \IPS\Theme::i()->getTemplate( 'view' )->videoOnline( $this->video );
	}
			 
	/**
	 * toggleFields
	 */	
	protected function toggleFields()
	{
		# Ouput
		if ( \IPS\Request::i()->isAjax() )
		{
			$sidebar = $this->video->container()->cfields_pos == 'sidebar' ? true : false;
			\IPS\Output::i()->json( array( 	'type' 	=> 'OK', 'html'	=> \IPS\Theme::i()->getTemplate( 'view' )->customFields( $this->video, $sidebar ) ) );
		}
		else
		{
			\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
		}		
	}
	
	/**
	 * Edit video
	 *
	 */
	protected function edit()
	{
		if ( !$this->video->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_edit', '2BIMVB110/5', 403 );
		}
		
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'front_video.js', 'videobox', 'front' ) );
		
		$class = static::$contentModel;
		
		$formElements = $class::formElements( $this->video, $this->video->container() );		
		$form = new \IPS\Helpers\Form( 'form', isset( \IPS\Member::loggedIn()->language()->words[ $class::$formLangPrefix . '_save' ] ) ? $class::$formLangPrefix . '_save' : 'save' );
		$form->class = 'ipsForm_vertical';
		
		foreach( $formElements as $name => $formContent )
		{
			$form->add( $formContent );
		}
		
		if ( $values = $form->values() )
		{
			$this->video->processForm( $values );
			$this->video->save();

			\IPS\Output::i()->redirect( $this->video->url() );
		}

		$formTemplate = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'post', 'videobox' ) ), 'postForm' ), $this->video->container(), TRUE );
		
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'videobox_editvideo' );
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'post' )->postPage( $this->video->container(), $formTemplate, $title );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( $title );
		\IPS\Output::i()->breadcrumb[] = array( $this->video->container()->url(), $this->video->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->video->url(), $this->video->mapped('title') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );
	}
	
	/**
	 * Manage video online
	 */	
	protected function manageSub()
	{
		if ( !$this->video->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_edit', '2BIMVB110/6', 403, '' );
		}
		
		if ( ! \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_post ) ) or \IPS\Settings::i()->vb_conf_online_sub != 1 )
		{
			\IPS\Output::i()->error( 'vb_error_suboff', '2BIMVB110/7', 403, '' );			
		}
		
		# Upload subtitles
		foreach( \IPS\Db::i()->select( '*', 'videobox_subtitles', array( "vbsub_video=?", $this->video->id ), 'vbsub_id DESC' ) as $row )
		{
			$subtitles[] = \IPS\File::get( 'videobox_Subtitles', $row['vbsub_url'] );
			$subtitles_[] = $row;
		}

		$form = new \IPS\Helpers\Form;		
		$form->add( new \IPS\Helpers\Form\Upload( 'videobox_uploadsub', $subtitles, FALSE, array( 
			'storageExtension' 	=> 'videobox_Subtitles', 
			'allowedFileTypes'	=> array( 'vtt' ), 
			'maxFileSize'		=> 2, 
			'multiple'			=> TRUE
		) ) );
		$form->class = 'ipsForm_vertical';	
		$form->action = $form->action->setQueryString( array( 'sub' => 1 ) );
		if ( $values = $form->values() )
		{
			if ( \IPS\Request::i()->sub == 1 )
			{
				$existing = iterator_to_array( \IPS\Db::i()->select( '*', 'videobox_subtitles', array( "vbsub_video=?", $this->video->id ) )->setKeyField( 'vbsub_url' ) );
		
				$files = array();
				foreach ( $values['videobox_uploadsub'] as $file )
				{
					$files[ $k ] = (string) $file;
					if ( !isset( $existing[ (string) $file ] ) )
					{
						\IPS\Db::i()->insert( 'videobox_subtitles', array(
							'vbsub_video'	=> $this->video->id,
							'vbsub_url'		=> (string) $file,
						) );
					}
					$k++;
					unset( $existing[ (string) $file ] );
				}

				foreach ( $existing as $location => $file )
				{
					try
					{
						\IPS\File::get( 'videobox_Subtitles', $location )->delete();
					}
					catch ( \Exception $e ) { }
					
					\IPS\Db::i()->delete( 'videobox_subtitles', array( 'vbsub_id=?', $file['vbsub_id'] ) );
				}
				
				\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=videobox&module=video&controller=view&do=manageSub&id={$this->video->id}" ) );
			}
		}

		# Output
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'videobox_manaSub' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->manageSub( $this->video, $form, $subtitles_ );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( $title );
		\IPS\Output::i()->breadcrumb[] = array( $this->video->container()->url(), $this->video->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->video->url(), $this->video->mapped('title') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );
	}

	protected function saveExternalVideos()
	{
		# Check permission
		if ( !$this->video->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_edit', '2BIMVB110/8', 404 );
		}
		
		if ( ! \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_post ) ) or \IPS\Settings::i()->vb_conf_online_sub != 1 )
		{
			\IPS\Output::i()->error( 'vb_error_suboff', '2BIMVB110/9', 404 );			
		}
		
		# Check server
		try
		{
			$server	= \IPS\videobox\Servers\Server::load( \IPS\Request::i()->server );
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_server_not_found', '2BIMVB110/A', 404 );
		}		
		
		$this->video->saveExternalVideos($server->id);
		//\IPS\Output::i()->redirect( $this->video->url()->setQueryString( array( 'area' => 'online', 'serverID' => $server->id, 'vid' => 1 ) ) );
		\IPS\Output::i()->redirect( $this->video->url()->setQueryString( array( 'do' => 'manageSub' ) ) );
	}
	
	protected function saveUploadVideos()
	{
		# Check permission
		if ( !$this->video->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_edit', '2BIMVB110/B', 404 );
		}
		
		if ( ! \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_post ) ) or \IPS\Settings::i()->vb_conf_online_sub != 1 )
		{
			\IPS\Output::i()->error( 'vb_error_suboff', '2BIMVB110/C', 404 );			
		}
		
		# Check server
		try
		{
			$server	= \IPS\videobox\Servers\Server::load( \IPS\Request::i()->server );
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_server_not_found', '2BIMVB110/D', 404 );
		}		

		# Save
		$sv = "server_" . $server->id;		
		$svarray = \IPS\Request::i()->$sv;
		
		$subtitle = "subtitle_" . $server->id;
		$subarray = \IPS\Request::i()->$subtitle;

		foreach( (array) $svarray as $k => $row )
		{
			if ( $row )
			{
				$data[$row] = $subarray[$k];
			}
		}
	
		if ( \count( $data  ) > 0 )
		{
			foreach( $data as $url => $sub )
			{
				\IPS\Db::i()->update( 'videobox_online', array( 'vbo_sub' => $sub ), array( 'vbo_url=? and vbo_video=? and vbo_server=?', $url, $this->video->id, $server->id ) );
			}
			\IPS\Output::i()->redirect( $this->video->url()->setQueryString( array( 'area' => 'online', 'serverID' => $server->id, 'vid' => 1 ) ) );
		}
		else
		{
			\IPS\Output::i()->redirect( $this->video->url() );
		}
	}

	//-----------------------------
	// Use to download fields
	//-----------------------------
	protected function downloadff()
	{
		$fid = \IPS\Request::i()->fid;

		try
		{
			$row = \IPS\Db::i()->select( '*', 'videobox_ccontent', 'video_id=' . $this->video->id )->first();
			
			$file = \IPS\File::get( 'videobox_VideoField', $row[ 'field_' . $fid ] );

			$bim = explode( "/" . $file->configuration['url'] . "/", (string) $file->url );
			
			$filepath = $file->configuration['dir'] . "/" . $bim[1];

			$name = explode(".", basename((string) $file->url));
			array_pop($name); // remove the last element in array, it's a long string
			$realname = implode(".", $name);
			$realname = str_replace("_", ".", $realname);
			$file->originalFilename = $realname;
			$this->_download( $file );		
		}
		catch ( \UnderflowException $e ) { }	
	}

	protected function _download( \IPS\File $file )
	{
        if ( !$file->filesize() )
        {
            \IPS\Output::i()->error( 'node_error', '2BIMVB110/10', 404 );
        }

		/* If it's an AWS file just redirect to it */
		if ( $file instanceof \IPS\File\Amazon )
		{
			\IPS\Output::i()->redirect( $file->generateTemporaryDownloadUrl() );
		}

		/* Print the file, honoring ranges */
		$ranges	= new \IPS\Http\Ranges( $file, 0 );

		/* If using PHP-FPM, close the request so that __destruct tasks are run after data is flushed to the browser
			@see http://www.php.net/manual/en/function.fastcgi-finish-request.php */
		if( function_exists( 'fastcgi_finish_request' ) )
		{
			fastcgi_finish_request();
		}

		exit;
	}
	
	//-----------------------------
	// Load uploaded video
	//-----------------------------
	protected function loadVideo()
	{
		/* No direct linking check */
		if ( !isset( \IPS\Request::i()->csrfKey ) || ! \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_groups ) ) )
		{
			\IPS\Output::i()->redirect( $this->video->url() );
		}
		
		\IPS\Session::i()->csrfCheck();

		/* Load video */
		try
		{
			$data = \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_id=?', (int) \IPS\Request::i()->play ) )->first();
		}
		catch( \UnderflowException $e ){}	

		$file = \IPS\File::get( 'videobox_Videos', $data['vbo_url'] );
		
		$ranges	= new \IPS\Http\Ranges( $file, 0 );
		
		/* If using PHP-FPM, close the request so that __destruct tasks are run after data is flushed to the browser
			@see http://www.php.net/manual/en/function.fastcgi-finish-request.php */
		if( function_exists( 'fastcgi_finish_request' ) )
		{
			fastcgi_finish_request();
		}

		exit;		
	}	
	
	//-----------------------------
	// add video parts
	//-----------------------------
	protected function addNewPart()
	{
		# Check permission
		if ( !$this->video->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_addedit', '2BIMVB110/E', 404 );
		}
		
		# Check server
		try
		{
			$server	= \IPS\videobox\Servers\Server::load( \IPS\Request::i()->server );
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_server_not_found', '2BIMVB110/F', 404 );
		}
		
		# Form
		$form = new \IPS\Helpers\Form;
		$form->class = 'ipsPad vbAddNewPart';
		$form->add( new \IPS\Helpers\Form\Stack( 'vb_part_url', array(), TRUE ) );
		
		# Save
		if ( $values = $form->values() )
		{	
			if ( isset($values['vb_part_url']) )
			{
				try
				{
					$lastPart = \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_upload=0 and vbo_server=? and vbo_video=?', $server->id, $this->video->id ), "vbo_id ASC" )->first();
				}
				catch ( \UnderflowException $e ){}
				
				foreach( $values['vb_part_url'] as $url )
				{
					\IPS\Db::i()->insert( 'videobox_online', array(
						'vbo_video'		=> $this->video->id,
						'vbo_url'		=> $url,					
						'vbo_server'	=> $server->id,
					) );				
				}
			}
			\IPS\Output::i()->redirect( $this->video->url()->setQueryString( array( 'area' => 'online', 'serverID' => $server->id, 'vid' => $pos + $count ) ) );
		}
		
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack( 'vb_addNewPart' );
		\IPS\Output::i()->output = $form;		
	}
	
	//-----------------------------
	// Manage Video Part
	//-----------------------------
	protected function manageNewPart()
	{
		# Check permission
		if ( !$this->video->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_addedit', '2BIMVB110/G', 404 );
		}

		# Check video part
		try
		{
			$part = \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_upload=0 and vbo_id=?', (int) \IPS\Request::i()->part ) )->first();
		}
		catch ( \UnderflowException $e )
		{
			\IPS\Output::i()->error( 'videobox_video_not_found', '2BIMVB110/H', 404 );
		}
				
		# Form
		$form = new \IPS\Helpers\Form;
		$form->class = 'ipsPad vbAddNewPart';
		$form->add( new \IPS\Helpers\Form\Select( 'vb_part_action', 'edit', TRUE, array(
			'options' => array( 'edit' => 'vb_part_edit', 'delete' => 'vb_part_delete' )
		) ) );
		$form->add( new \IPS\Helpers\Form\Url( 'vb_part_url', $part['vbo_url'], TRUE ) );
		
		# Save
		if ( $values = $form->values() )
		{
			$pos = 0;
			
			if ( $values['vb_part_action'] == 'edit' )
			{
				\IPS\Db::i()->update( 'videobox_online', array( 'vbo_url' => $values['vb_part_url'] ), array( 'vbo_id=?', $part['vbo_id'] ) );
				\IPS\Output::i()->redirect( $this->video->url()->setQueryString( array( 'area' => 'online', 'serverID' => $part['vbo_server'], 'vid' => $part['vbo_position'] ) ) );
			}
			else
			{
				\IPS\Db::i()->delete( 'videobox_online', array( 'vbo_id=?', $part['vbo_id'] ) );
				\IPS\Output::i()->redirect( $this->video->url()->setQueryString( array( 'area' => 'online', 'serverID' => $part['vbo_server'], 'vid' => 1 ) ) );
			}
			
		}
		
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack( 'vb_managePart' );
		\IPS\Output::i()->output = $form;
	}
	
	/*-------------------------------------------------------------------------*/
	// BUY
	/*-------------------------------------------------------------------------*/	
	protected function buy()
	{
		\IPS\Session::i()->csrfCheck();
		
		/* Can we buy? */
		if ( !$this->video->canBuy() )
		{
			\IPS\Output::i()->error( 'no_module_permission', '2BIMVB110/J', 404 );
		}
		
		/* Create the item */		
		$price = $this->video->price();
		$item = new \IPS\videobox\extensions\nexus\Item\Video( $this->video->title, $price );
		$item->id = $this->video->id;
		
		try
		{
			$item->tax = $this->video->container()->nexus_tax ? \IPS\nexus\Tax::load( $this->video->container()->nexus_tax ) : NULL;
		}
		catch ( \OutOfRangeException $e ) { }
		if ( $this->video->container()->nexus_gateways )
		{
			$item->paymentMethodIds = explode( ',', $this->video->container()->nexus_gateways );
		}

		$item->payTo = $this->video->author();
		$item->commission = $this->video->container()->nexus_percent;
		if ( $fees = json_decode( $this->video->container()->nexus_transfee, TRUE ) )
		{
			$item->fee = new \IPS\nexus\Money( $fees[ $price->currency ]['amount'], $price->currency );
		}
				
		/* Generate the invoice */
		$invoice = new \IPS\nexus\Invoice;
		$invoice->member = \IPS\nexus\Customer::loggedIn();
		$invoice->addItem( $item );
		$invoice->return_uri = "app=videobox&module=videobox&controller=view&id={$this->video->id}";
		$invoice->save();
		
		/* Take them to it */
		\IPS\Output::i()->redirect( $invoice->url() );
	}
	
	/**
	 * React
	 *
	 * @return	void
	 */
	protected function react()
	{
		if( static::$contentModel::checkPointsDB() )
		{
			$newPoints = $this->video->container()->points_on == 1 ? $this->video->container()->points_reaction : \IPS\Settings::i()->vb_conf_points_reaction;
		
			if ( $newPoints > 0 )
			{
				$member = $member ?: \IPS\Member::loggedIn();
				$reacted = $this->video->reacted( $member );
				
				if ( $reacted )
				{
					if ( \IPS\Settings::i()->vb_conf_points_decrease == 1 )
					{
						\IPS\videobox\Video::updatePoints( $this->video->author()->member_id, $newPoints, true );
					}
				}
				else
				{
					\IPS\videobox\Video::updatePoints( $this->video->author()->member_id, $newPoints );
				}
			}
		}
		
		parent::react();
	}	
	
}