<?php


namespace IPS\videobox\Collections;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Custom Profile Field Group Node
 */
class _Collection extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_collections';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'vcol_';
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'order';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'videobox_collections';
	
	/**
	 * @brief	[Node] Subnode class
	 */
	public static $subnodeClass = NULL;
	
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'videobox',
		'module'	=> 'video',
		'prefix'	=> 'collections_',
		'all'		=> 'collections_manage',
	);

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'videobox_collection_';

	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Translatable( 'vcol_title', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $this->id ? "videobox_collection_{$this->id}" : NULL ) ) ) );
	}
	
	/**
	 * Get URL
	 *
	 * @return	\IPS\Http\Url
	 */
	public function url()
	{
		return \IPS\Http\Url::internal( "app=videobox&module=video&controller=collection&id={$this->_id}", 'front', 'videobox_collection', $this->title_furl );
	}	
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}
		
		/* Save lang for title */
		foreach ( array( 'vcol_title' => "videobox_collection_{$this->id}" ) as $fieldKey => $langKey )
		{
			if ( isset( $values[ $fieldKey ] ) )
			{
				\IPS\Lang::saveCustom( 'videobox', $langKey, $values[ $fieldKey ] );
				
				if ( $fieldKey === 'vcol_title' )
				{
					$this->title = $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ];					
					$this->title_furl = \IPS\Http\Url::seoTitle( $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ] );
				}
				
				unset( $values[ $fieldKey ] );
			}
		}	
		
		parent::saveForm( $values );			
	}
		
	/**
	 * [Node] Does the currently logged in user have permission to edit permissions for this node?
	 *
	 * @return	bool
	 */
	public function canManagePermissions()
	{
		return false;
	}
	
	/**
	 * [ActiveRecord] Duplicate
	 *
	 * @return	void
	 */
	public function __clone()
	{
		$oldId = $this->id;
		parent::__clone();
		\IPS\Lang::saveCustom( 'videobox', "videobox_collection_{$this->id}", iterator_to_array( \IPS\Db::i()->select( 'word_custom, lang_id', 'core_sys_lang_words', array( 'word_key=?', "videobox_collection_{$oldId}" ) )->setKeyField( 'lang_id' )->setValueField('word_custom') ) );
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		\IPS\Lang::deleteCustom( 'videobox', 'videobox_collection_' . $this->id );
		\IPS\Lang::deleteCustom( 'core', 'videobox_collection_' . $this->id );  // Remove custom lang for cloned catergory, because it was saved in core app	
	}
}