<?php

namespace IPS\videobox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * page active record
 */
class _Page extends \IPS\Patterns\ActiveRecord
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_pages';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'vbp_';

	/**
	 * Get Lang
	 */
	public function get__title()
	{
		if ( \IPS\Member::loggedIn()->language()->checkKeyExists( 'videobox_page_' . $this->id ) )
		{
			return \IPS\Member::loggedIn()->language()->addToStack('videobox_page_' . $this->id);
		}
	}
	
	public function get__metakeywords()
	{
		if ( \IPS\Member::loggedIn()->language()->checkKeyExists( 'videobox_page_' . $this->id . '_metakeywords' ) )
		{
			return \IPS\Member::loggedIn()->language()->addToStack( 'videobox_page_' . $this->id . '_metakeywords' );
		}
	}

	public function get__metadescription()
	{
		if ( \IPS\Member::loggedIn()->language()->checkKeyExists( 'videobox_page_' . $this->id . '_metadescription' ) )
		{
			return \IPS\Member::loggedIn()->language()->addToStack( 'videobox_page_' . $this->id . '_metadescription' );
		}
	}
	
	public function get__content()
	{
		if ( \IPS\Member::loggedIn()->language()->checkKeyExists( 'videobox_page_' . $this->id . '_content' ) )
		{
			return \IPS\Member::loggedIn()->language()->addToStack( 'videobox_page_' . $this->id . '_content' );
		}
	}	
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		\IPS\File::unclaimAttachments( 'videobox_Pages', $page->id );		
		\IPS\Db::i()->delete( 'videobox_pages', array( 'vbp_id=?', $this->id ) );
		
		$lang = array(
			'vbp_title' 			=> 'videobox_page_' . $this->id, 
			'vbp_metakeywords' 		=> 'videobox_page_' . $this->id . '_metakeywords',
			'vbp_metadescription' 	=> 'videobox_page_' . $this->id . '_metadescription',
			'vbp_content' 			=> 'videobox_page_' . $this->id . '_content',
		);
		
		foreach ( $lang as $fieldKey => $langKey )
		{
			\IPS\Lang::deleteCustom( 'videobox', $langKey );
		}
		
		return parent::delete();
	}
	
	/**
	 * [ActiveRecord] Enable Toggle
	 *
	 * @return	void
	 */
	public function enableToggle($open)
	{
		$status = $open != 1 ? 1 : 0;
		\IPS\Db::i()->update( 'videobox_pages', array( 'vbp_open' => $status ), 'vbp_id=' . (int) \IPS\Request::i()->id );
	}	
}