<?php


namespace IPS\videobox\QuickSearch;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Custom Profile Field Group Node
 */
class _Group extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_quicksearchGroup';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'v_qs_';
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'order';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'videobox_quicksearch';
	
	/**
	 * @brief	[Node] Subnode class
	 */
	public static $subnodeClass = 'IPS\videobox\QuickSearch\Term';
	
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'videobox',
		'module'	=> 'video',
		'prefix'	=> 'quicksearch_',
		'all'		=> 'quicksearch_manage',
	);

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'videobox_qsearchgroups_';

	/**
	 * Get custom fields
	 *
	 * @return	array
	 */
	protected function get_cfields()
	{
		$fields = array();
		foreach ( explode( ',', $this->_data['cfields'] ) as $fieldId )
		{
			try
			{
				$fields[ $fieldId ] = \IPS\videobox\Field::load( $fieldId );
			}
			catch ( \OutOfRangeException $e ) { }
		}

		return $fields;
	}
	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Translatable( 'v_qs_title', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $this->id ? "videobox_qsearchgroups_{$this->id}" : NULL ) ) ) );
		$customFields = array();
		foreach ( \IPS\Db::i()->select( 'cf_id', 'videobox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "videobox_field_{$fieldId}" );
		}		
		if ( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'v_qs_field', $this->field, TRUE, array( 'options' => $customFields, 'multiple' => FALSE) ) );
		}
		$form->add( new \IPS\Helpers\Form\Select( 'v_qs_mode', $this->id ? $this->mode : 'equal', TRUE, array(
            'options' => array(
	            'like'   	=> 'v_qs_mode_like',			
	            'equal'   	=> 'v_qs_mode_equal',
	            'greater'   => 'v_qs_mode_greater',
	            'lessthan'  => 'v_qs_mode_lessthan',
	            'different'	=> 'v_qs_mode_different',					
            )
        ) ) );
		$form->add( new \IPS\Helpers\Form\Select( 'v_qs_canuse', ( $this->canuse != '*' ) ? explode( ",", $this->canuse ) : $this->canuse, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE, 'unlimited' => '*', 'unlimitedLang' => 'everyone' ) ) );
		
		$form->add( new \IPS\Helpers\Form\Node( 'v_qs_cat', ( $this->id and $this->cat != '*' ) ? $this->cat : 0, FALSE, array(
			'class'           => '\IPS\videobox\Category',
			'multiple'        => true,
			'zeroVal'         => 'all',
		) ) );
		
	}
	
	/**
	 * Can search
	 *
	 */
	public function canUse( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		if ( $this->canuse == '*' or \IPS\Member::loggedIn()->inGroup( explode(",", $this->canuse) ) )
		{
			return TRUE;
		}
		else return FALSE;
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}
	
		if( isset( $values['v_qs_cat'] ) )
		{
			if ( $values['v_qs_cat'] == 0 )
			{
				$values['v_qs_cat'] = '*';
			}
			else 
			{
				$cats = array();
				foreach ( $values['v_qs_cat'] as $cat )
				{
					$cats[] = $cat->_id;
				}
				
				$values['v_qs_cat'] = ( implode( ',', $cats ) );
			}
		}
		
		\IPS\Lang::saveCustom( 'videobox', "videobox_qsearchgroups_{$this->id}", $values['v_qs_title'] );

		parent::saveForm( $values );		
	}
		
	/**
	 * [Node] Does the currently logged in user have permission to edit permissions for this node?
	 *
	 * @return	bool
	 */
	public function canManagePermissions()
	{
		return false;
	}
	
	/**
	 * [ActiveRecord] Duplicate
	 *
	 * @return	void
	 */
	public function __clone()
	{
		$oldId = $this->id;
		parent::__clone();
		\IPS\Lang::saveCustom( 'videobox', "videobox_qsearchgroups_{$this->id}", iterator_to_array( \IPS\Db::i()->select( 'word_custom, lang_id', 'core_sys_lang_words', array( 'word_key=?', "videobox_qsearchgroups_{$oldId}" ) )->setKeyField( 'lang_id' )->setValueField('word_custom') ) );
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		\IPS\Lang::deleteCustom( 'videobox', 'videobox_qsearchgroups_' . $this->id );
		\IPS\Lang::deleteCustom( 'core', 'videobox_qsearchgroups_' . $this->id );  // Remove custom lang for cloned catergory, because it was saved for core.		
	}

	/**
	 * [Node] Get buttons to display in tree
	 * Example code explains return value
	 *
	 * @code
	 	array(
	 		array(
	 			'icon'	=>	'plus-circle', // Name of FontAwesome icon to use
	 			'title'	=> 'foo',		// Language key to use for button's title parameter
	 			'link'	=> \IPS\Http\Url::internal( 'app=foo...' )	// URI to link to
	 			'class'	=> 'modalLink'	// CSS Class to use on link (Optional)
	 		),
	 		...							// Additional buttons
	 	);
	 * @encode
	 * @param	string	$url		Base URL
	 * @param	bool	$subnode	Is this a subnode?
	 * @return	array
	 */
	public function getButtons( $url, $subnode=FALSE )
	{
		$buttons	= parent::getButtons( $url, $subnode );

		$buttons['add']['title']	= 'videobox_qsearch_add';

		return $buttons;
	}
}