<?php

namespace IPS\videobox\QuickSearch;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Custom Profile Field Node
 */
class _Term extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_quicksearchTerm';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'v_qst_';
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
	
	/**
	 * @brief	[Node] Parent Node ID Database Column
	 */
	public static $parentNodeColumnId = 'group';
	
	/**
	 * @brief	[Node] Parent Node Class
	 */
	public static $parentNodeClass = 'IPS\videobox\QuickSearch\Group';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'order';
	
	
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'videobox',
		'module'	=> 'video',
		'prefix'	=> 'quicksearch_',
		'all'		=> 'quicksearch_manage',
	);

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'videobox_qsearchterms_';

	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Translatable( 'v_qst_title', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $this->id ? "videobox_qsearchterms_{$this->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'v_qst_value', $this->id ? $this->value : "", TRUE, array() ) );		
	}
	
	/**
	 * Get URL
	 *
	 * @return	\IPS\Http\Url
	 */
	public function url()
	{
		return \IPS\Http\Url::internal( "app=videobox&module=video&controller=quicksearch&id={$this->_id}", 'front', 'videobox_quicksearch', $this->title_furl );
	}		
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}
		
		/* Save lang for title */
		foreach ( array( 'v_qst_title' => "videobox_qsearchterms_{$this->id}" ) as $fieldKey => $langKey )
		{
			if ( isset( $values[ $fieldKey ] ) )
			{
				\IPS\Lang::saveCustom( 'videobox', $langKey, $values[ $fieldKey ] );
				
				if ( $fieldKey === 'v_qst_title' )
				{
					$this->title = $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ];					
					$this->title_furl = \IPS\Http\Url::seoTitle( $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ] );
				}
				
				unset( $values[ $fieldKey ] );
			}
		}	
		
		parent::saveForm( $values );		
	}
		
	/**
	 * [Node] Does the currently logged in user have permission to edit permissions for this node?
	 *
	 * @return	bool
	 */
	public function canManagePermissions()
	{
		return false;
	}
	
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		\IPS\Lang::deleteCustom( 'videobox', 'videobox_qsearchterms_' . $this->id );
		\IPS\Lang::deleteCustom( 'core', 'videobox_qsearchterms_' . $this->id );  // Remove custom lang for cloned catergory, because it was saved for core.
	}

}