<?php

namespace IPS\videobox\Servers;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Server active record
 */
class _Server extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_servers';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'vbs_';
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'videobox_servers';
	
	/**
	 * @brief	[Node] Subnode class
	 */
	public static $subnodeClass = NULL;
	
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'videobox',
		'module'	=> 'video',
		'prefix'	=> 'servers_',
		'all'		=> 'servers_manage',
	);


	/**
	 * Get title of category
	 *
	 * @return	string
	 */
	protected function get__title()
	{
		return $this->name;
	}
	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Text( 'vbs_name', $this->name ? $this->name : NULL, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Select( 'vbs_groups', $this->groups ? array_filter( explode( ',', $this->groups ) ) : 0, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE, 'unlimited' => 0 ) ) );
		$form->add( new \IPS\Helpers\Form\Select( 'vbs_upload', $this->upload ? $this->upload : 0, FALSE, array( 'options' => array( '0' => 'vbs_external', '1' => 'vbs_uploadvideo' ) ) ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vbs_not_null', $this->not_null ? $this->not_null : 0, FALSE ) );		
	}
	
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}

		parent::saveForm( $values );			
	}
		
	/**
	 * [Node] Does the currently logged in user have permission to edit permissions for this node?
	 *
	 * @return	bool
	 */
	public function canManagePermissions()
	{
		return false;
	}

	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{		
		parent::delete();
		\IPS\Db::i()->delete( 'videobox_servers', array( 'vbs_id=?', $this->id ) );
		\IPS\Db::i()->delete( 'videobox_online', array( 'vbo_server=?', $this->id ) );		
	}
}