<?php
namespace IPS\videobox\Video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Video Comment Model
 */
class _Comment extends \IPS\Content\Comment implements \IPS\Content\EditHistory, \IPS\Content\Hideable, \IPS\Content\Searchable, \IPS\Content\Embeddable, \IPS\Content\Shareable
{
	use \IPS\Content\Reactable, \IPS\Content\Reportable;
	
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
		
	
	/**
	 * @brief	[Content\Comment]	Item Class
	 */
	public static $itemClass = 'IPS\videobox\Video';
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_comments';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'vcom_';
	
	/**
	 * @brief	Application
	 */
	public static $application = 'videobox';
	
	/**
	 * @brief	Title
	 */
	public static $title = 'videobox_video_comment';
	
	/**
	 * @brief	Database Column Map
	 */
	public static $databaseColumnMap = array(
		'item'				=> 'videoID',
		'author'			=> 'mid',
		'author_name'		=> 'author',
		'content'			=> 'text',
		'date'				=> 'date',
		'ip_address'		=> 'ip_address',
		'edit_time'			=> 'edit_time',
		'edit_member_name'	=> 'edit_name',
		'edit_show'			=> 'append_edit',
		'approved'			=> 'open'
	);
	
	/**
	 * @brief	Icon
	 */
	public static $icon = 'play-circle';
	
	/**
	 * @brief	Reputation Type
	 */
	public static $reputationType = 'comment_id';
	
	/**
	 * @brief	[Content]	Key for hide reasons
	 */
	public static $hideLogKey = 'videobox-videos';
	
	/**
	 * Get URL for doing stuff
	 *
	 * @param	string|NULL		$action		Action
	 * @return	\IPS\Http\Url
	 */
	public function url( $action='find' )
	{
		return parent::url( $action )->setQueryString( 'tab', 'comments' );
	}

	/**
	 * Get snippet HTML for search result display
	 *
	 * @param	array		$indexData		Data from the search index
	 * @param	array		$authorData		Basic data about the author. Only includes columns returned by \IPS\Member::columnsForPhoto()
	 * @param	array		$itemData		Basic data about the item. Only includes columns returned by item::basicDataColumns()
	 * @param	array|NULL	$containerData	Basic data about the author. Only includes columns returned by container::basicDataColumns()
	 * @param	array		$reputationData	Array of people who have given reputation and the reputation they gave
	 * @param	int|NULL	$reviewRating	If this is a review, the rating
	 * @param	string		$view			'expanded' or 'condensed'
	 * @return	callable
	 */
	public static function searchResultSnippet( array $indexData, array $authorData, array $itemData, array $containerData = NULL, array $reputationData, $reviewRating, $view )
	{
		if ( $itemData['v_livestream_is'] == 1 && $itemData['v_livestream_data'] )
		{
			$lst = @json_decode($itemData['v_livestream_data']);
			$screenshot = $lst->thumbnail;		
		}
		else
		{
			if ($itemData['v_thumbnail'])
			{
				$screenshot = $itemData['v_thumbnail_medium'];
			}
			else
			{
				if ($itemData['v_externalThumb'])
				{
					$screenshot = $itemData['v_externalThumb'];
				}
				else
				{
					$screenshot = \IPS\Application::load('videobox')->getthumbnail( $itemData['v_content'] ) ? \IPS\Application::load('videobox')->getthumbnail( $itemData['v_content'] )['small'] : (string) \IPS\Theme::i()->resource( 'videoNoThumb.jpg', 'videobox', 'front' );
				}
			}
		}
		
		$url = \IPS\Http\Url::internal( \IPS\videobox\Video::$urlBase . $indexData['index_item_id'], 'front', \IPS\videobox\Video::$urlTemplate, \IPS\Http\Url::seoTitle( $indexData['index_title'] ?: $itemData[ \IPS\videobox\Video::$databasePrefix . \IPS\videobox\Video::$databaseColumnMap['title'] ] ) );
		
		return \IPS\Theme::i()->getTemplate( 'global', 'videobox', 'front' )->searchResultCommentSnippet( $indexData, $screenshot, $url, $reviewRating, $view == 'condensed' );
	}	

	/**
	 * Reaction Type
	 *
	 * @return	string
	 */
	public static function reactionType()
	{
		return 'vcom_id';
	}	
	
	# POINTS
	public function postCreate()
	{
		parent::postCreate();

		if ( \IPS\Settings::i()->vb_conf_points_on == 1 )
		{
			$newPoints = $this->item()->container()->points_on == 1 ? $this->item()->container()->points_comment : \IPS\Settings::i()->vb_conf_points_comment;

			if ( $newPoints == -1 )
			{
				return;
			}
			
			if( $this->item()->checkPointsDB() )
			{		
				$this->item()->updatePoints( $this->author()->member_id, $newPoints );
			}		
		}
	}	
	
	/**
	 * Delete Comment
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		
		// Points
		if ( \IPS\Settings::i()->vb_conf_points_on == 1 && \IPS\Settings::i()->vb_conf_points_decrease == 1 )
		{		
			$newPoints = $this->item()->container()->points_on == 1 ? $this->item()->container()->points_comment : \IPS\Settings::i()->vb_conf_points_comment;

			if ( $newPoints == -1 )
			{
				return;
			}
						
			if( $this->item()->checkPointsDB() )
			{		
				$this->item()->updatePoints( $this->author()->member_id, $newPoints, true );
			}			
		}		
	}
}