<?php

namespace IPS\videobox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Video
 */
class _Video extends \IPS\Content\Item implements
\IPS\Content\Permissions,
\IPS\Content\Tags,
\IPS\Content\Ratings,
\IPS\Content\Followable,
\IPS\Content\ReadMarkers,
\IPS\Content\Views,
\IPS\Content\Hideable, \IPS\Content\Featurable, \IPS\Content\Pinnable, \IPS\Content\Lockable,
\IPS\Content\Shareable,
\IPS\Content\Searchable,
\IPS\Content\Embeddable,
\IPS\Content\MetaData
{
	use \IPS\Content\Reactable, \IPS\Content\Reportable, \IPS\Content\ItemTopic
	{
		\IPS\Content\ItemTopic::changeAuthor as topicChangeAuthor;
	}
	
	/**
	 * @brief	Application
	 */
	public static $application = 'videobox';
	
	/**
	 * @brief	Module
	 */
	public static $module = 'video';
	
	/**
	 * @brief	Database Table
	 */
	public static $databaseTable = 'videobox_videos';
	
	/**
	 * @brief	Database Prefix
	 */
	public static $databasePrefix = 'v_';
	
	/**
	 * @brief	Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	Default Values
	 */
	protected static $defaultValues = NULL;
	
	/**
	 * @brief	Node Class
	 */
	public static $containerNodeClass = 'IPS\videobox\Category';
	
	/**
	 * @brief	Comment Class
	 */
	public static $commentClass = 'IPS\videobox\Video\Comment';
	
	/**
	 * @brief	Review Class
	 */
	public static $reviewClass = 'IPS\videobox\Video\Review';
		
	/**
	 * Set custom posts per page setting
	 *
	 * @return int
	 */
	public static function getCommentsPerPage()
	{
		return 15;
	}	

	/**
	 * @brief	Database Column Map
	 */
	public static $databaseColumnMap = array(
		'container'				=> 'cat',
		'author'				=> 'author',
		'views'					=> 'views',
		'title'					=> 'title',
		'content'				=> 'content',
		'num_comments'			=> 'comments',
		'unapproved_comments'	=> 'unapproved_comments',
		'hidden_comments' 		=> 'hidden_comments',		
		'num_reviews'			=> 'reviews',
		'unapproved_reviews' 	=> 'unapproved_reviews',
		'hidden_reviews'		=> 'hidden_reviews',		
		'last_comment'			=> 'last_comment',
		'last_review'			=> 'last_review',		
		'date'					=> 'started',
		'updated'				=> 'updated',
		'rating'				=> 'rating',		
		'rating_average'		=> 'rating_average',
		'rating_total'			=> 'rating_total',
		'rating_hits'			=> 'ratings_count',
		'approved'				=> 'open',
		'approved_by'			=> 'approver',
		'approved_date'			=> 'approvedon',
		'pinned'				=> 'pinned',
		'featured'				=> 'featured',
		'locked'				=> 'locked',
		'ip_address'			=> 'ipaddress',
		'meta_data'				=> 'meta_data',
		'online'				=> 'livestream_online'
	);
	
	/**
	 * @brief	Hover preview
	 */
	public $tableHoverUrl = TRUE;
	
	/**
	 * @brief	Title
	 */
	public static $title = 'videobox_video';
	
	/**
	 * @brief	Icon
	 */
	public static $icon = 'play-circle';
	
	/**
	 * @brief	Form Lang Prefix
	 */
	public static $formLangPrefix = 'videobox_';
	
	
	/**
	 * @brief	Reputation Type
	 */
	public static $reputationType = 'v_id';
	
	/**
	 * @brief	[Content]	Key for hide reasons
	 */
	public static $hideLogKey = 'videobox-video';
	
	/**
	 * Columns needed to query for search result / stream view
	 *
	 * @return	array
	 */
	public static function basicDataColumns()
	{
		$return = parent::basicDataColumns();
		$return[] = 'v_thumbnail';
		$return[] = 'v_thumbnail_medium';
		$return[] = 'v_externalThumb';
		$return[] = 'v_views';
		$return[] = 'v_livestream_is';
		$return[] = 'v_livestream_data';
		$return[] = 'v_content';
		return $return;
	}
	
	
	/**
	 * Set title
	 *
	 * @param	string	$title	Title
	 * @return	void
	 */
	public function set_title( $title )
	{
		$this->_data['title'] = $title;
		$this->_data['title_furl'] = \IPS\Http\Url::seoTitle( $title );
	}	

	/**
	 * Get SEO name
	 *
	 * @return	string
	 */
	public function get_name_furl()
	{
		if( !$this->_data['title_furl'] )
		{
			$this->title_furl	= \IPS\Http\Url\Friendly::seoTitle( $this->title );
			$this->save();
		}

		return $this->_data['title_furl'] ?: \IPS\Http\Url\Friendly::seoTitle( $this->title );
	}
	
	/**
	 * @brief	Cached URLs
	 */
	protected $_url	= array();
	
	/**
	 * @brief	URL Base
	 */
	public static $urlBase = 'app=videobox&module=video&controller=view&id=';
	
	/**
	 * @brief	URL Base
	 */
	public static $urlTemplate = 'videobox_video';
	
	/**
	 * @brief	SEO Title Column
	 */
	public static $seoTitleColumn = 'title_furl';
	

	/**
	 * Get URL for last comment page
	 *
	 * @return	\IPS\Http\Url
	 */
	public function lastCommentPageUrl()
	{
		return parent::lastCommentPageUrl()->setQueryString( 'tab', 'comments' );
	}
	
	/**
	 * Get URL for last review page
	 *
	 * @return	\IPS\Http\Url
	 */
	public function lastReviewPageUrl()
	{
		return parent::lastCommentPageUrl()->setQueryString( 'tab', 'reviews' );
	}
	
	/**
	 * Can view?
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for or NULL for the currently logged in member
	 * @return	bool
	 */
	public function canView( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		if ( !$this->container()->open and !$member->isAdmin() )
		{
			return FALSE;
		}
		return parent::canView( $member );
	}
	
	/**
	 * Can edit?
	 * Authors can always edit their own files
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canEdit( $member=NULL )
	{
		return parent::canEdit( $member );
	}
	
	/**
	 * Can Rate?
	 *
	 * @param	\IPS\Member|NULL		$member		The member to check for (NULL for currently logged in member)
	 * @return	bool
	 * @throws	\BadMethodCallException
	 */
	public function canRate( \IPS\Member $member = NULL )
	{
		return $this->container()->bitoptions['rating'] and parent::canRate( $member );
	}	

	/**
	 * Can change author?
	 *
	 * @param	\IPS\Member\NULL	$member	The member (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canChangeAuthor( \IPS\Member $member = NULL )
	{
		return static::modPermission( 'edit', $member, $this->container() );
	}

	/**
	 * Change Author
	 *
	 * @param	\IPS\Member	$newAuthor	The new author
	 * @return	void
	 */
	public function changeAuthor( \IPS\Member $newAuthor, $log=TRUE )
	{
		parent::changeAuthor( $newAuthor );
		$this->topicChangeAuthor( $newAuthor, $log );
	}
	
	/**
	 * Get thumbnail ratio
	 */
	public function get_thumbratio()
	{
		$size = explode(":", \IPS\Settings::i()->vb_conf_thumbnail_medium);
		$radio = round( ($size[1]/$size[0]), 2 );
		return $radio * 100;
	}
	
	public function get_bigthumbratio()
	{
		$size = explode(":", \IPS\Settings::i()->vb_conf_featured_size);
		$radio = round( ($size[1]/$size[0]), 2 );
		return $radio * 100;
	}
	
	/**
	 * @brief	Can play?
	 */
	protected $canPlay = NULL;
	
	/**
	 * Can the member play this video?
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	bool
	 */
	public function canPlay( \IPS\Member $member = NULL )
	{
		if ( $this->canPlay === NULL )
		{
			try
			{
				$this->playCheck( $member );
				$this->canPlay = TRUE;
			}
			catch ( \DomainException $e )
			{
				$this->canPlay = FALSE;
			}
		}
		
		return $this->canPlay;
	}
	
	/**
	 * Play check
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	void
	 * @throws	\DomainException
	 */
	public function playCheck( \IPS\Member $member = NULL )
	{
		/* Basic permission check */
		$member = $member ?: \IPS\Member::loggedIn();
		if ( \IPS\Settings::i()->vb_conf_online_on != 1 || ( \IPS\Settings::i()->vb_conf_online_cats != '0' && !\in_array( $this->container()->id, explode( ",", \IPS\Settings::i()->vb_conf_online_cats ) ) ) )
		{
			throw new \DomainException( 'vb_playerOff' );
		}
		
		if ( !$member->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_groups ) ) )
		{
			if ( !$member->member_id )
			{
				throw new \DomainException( 'vb_guestcannotwatch' );
			}
			else
			{
				throw new \DomainException( 'vb_cannotwatch' );
			}
		}
		
		/* Paid? */
		if ( $this->container()->nexus_on and $this->cost and $this->price() and $member->member_id != $this->author()->member_id )
		{
			if ( !\count( \IPS\videobox\extensions\nexus\Item\Video::getPurchases( \IPS\nexus\Customer::load( $member->member_id ), $this->id, FALSE ) ) )
			{
				throw new \DomainException( 'video_not_purchased' );
			}
		}
		
		/* Video available to play or not */
		if ( !$this->hasVideoOnline() )
		{
			throw new \DomainException( 'videobox_error_novideo' );
		}		
	}	
	
	/*-------------------------------------------------------------------------*/
	// hasVideoOnline
	/*-------------------------------------------------------------------------*/
	public function hasVideoOnline()
	{
		if ( $this->livestream_is == 1 && $this->livestream )
		{
			return true;
		}
		else
		{
			$data = $this->videoOnline();
			if ( \count($data) > 0 )
			{
				foreach ( $data as $name => $row )
				{
					if ( \count($row['online']) > 0 && ( $row['groups'] == 0 || \IPS\Member::loggedIn()->inGroup( explode( ",", $row['groups'] ) ) ) )
					{
						return true;
					}
				}
			}
		}
		return false;
	}	
	/**
	 * Get snippet HTML for search result display
	 *
	 * @param	array		$indexData		Data from the search index
	 * @param	array		$authorData		Basic data about the author. Only includes columns returned by \IPS\Member::columnsForPhoto()
	 * @param	array		$itemData		Basic data about the item. Only includes columns returned by item::basicDataColumns()
	 * @param	array|NULL	$containerData	Basic data about the author. Only includes columns returned by container::basicDataColumns()
	 * @param	array		$reputationData	Array of people who have given reputation and the reputation they gave
	 * @param	int|NULL	$reviewRating	If this is a review, the rating
	 * @param	string		$view			'expanded' or 'condensed'
	 * @return	callable
	 */
	public static function searchResultSnippet( array $indexData, array $authorData, array $itemData, array $containerData = NULL, array $reputationData, $reviewRating, $view )
	{
		if ( $itemData['v_livestream_is'] == 1 && $itemData['v_livestream_data'] )
		{
			$lst = @json_decode($itemData['v_livestream_data']);
			$screenshot = $lst->thumbnail;		
		}
		else
		{
			if ($itemData['v_thumbnail'])
			{
				$screenshot = $itemData['v_thumbnail_medium'];
			}
			else
			{
				if ($itemData['v_externalThumb'])
				{
					$screenshot = $itemData['v_externalThumb'];
				}
				else
				{
					$screenshot = \IPS\Application::load('videobox')->getthumbnail( $itemData['v_content'] ) ? \IPS\Application::load('videobox')->getthumbnail( $itemData['v_content'] )['small'] : (string) \IPS\Theme::i()->resource( 'videoNoThumb.jpg', 'videobox', 'front' );
				}
			}
		}

		$url = \IPS\Http\Url::internal( static::$urlBase . $indexData['index_item_id'], 'front', static::$urlTemplate, \IPS\Http\Url::seoTitle( $indexData['index_title'] ?: $itemData[ static::$databasePrefix . static::$databaseColumnMap['title'] ] ) );

		return \IPS\Theme::i()->getTemplate( 'global', 'videobox', 'front' )->searchResultVideoSnippet( $indexData, $itemData, $screenshot, $url, $view == 'condensed' );
	}
	
	/**
	 * Get items with permisison check
	 *
	 * @param	array		$where				Where clause
	 * @param	string		$order				MySQL ORDER BY clause (NULL to order by date)
	 * @param	int|array	$limit				Limit clause
	 * @param	string		$permissionKey		A key which has a value in the permission map (either of the container or of this class) matching a column ID in core_permission_index
	 * @param	bool|NULL	$includeHiddenItems	Include hidden files? Boolean or NULL to detect if currently logged member has permission
	 * @param	int			$queryFlags			Select bitwise flags
	 * @param	\IPS\Member	$member				The member (NULL to use currently logged in member)
	 * @param	bool		$joinContainer		If true, will join container data (set to TRUE if your $where clause depends on this data)
	 * @param	bool		$joinComments		If true, will join comment data (set to TRUE if your $where clause depends on this data)
	 * @param	bool		$joinReviews		If true, will join review data (set to TRUE if your $where clause depends on this data)
	 * @return	\IPS\Patterns\ActiveRecordIterator|int
	 */
	public static function getItemsWithPermission( $where=array(), $order=NULL, $limit=10, $permissionKey='read', $includeHiddenItems=\IPS\Content\Hideable::FILTER_AUTOMATIC, $queryFlags=0, \IPS\Member $member=NULL, $joinContainer=FALSE, $joinComments=FALSE, $joinReviews=FALSE, $countOnly=FALSE, $joins=NULL, $skipPermission=FALSE, $joinTags=TRUE, $joinAuthor=TRUE, $joinLastCommenter=TRUE, $showMovedLinks=FALSE )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		if ( !$member->isAdmin() )
		{
			$where[] = array( 'videobox_categories.vcat_open=1' );
			$joinContainer = TRUE;
		}
		
		$joins[] = array(
			'from'		=> array( "videobox_ccontent", "field" ),
			'where'		=> "field.video_id=videobox_videos.v_id",
		);
	
		if ( \IPS\Settings::i()->tags_enabled && \IPS\Request::i()->tagsfilter )
		{
			$itemClass = \get_called_class();
			$idColumn = static::$databasePrefix . static::$databaseColumnId;			
			$joins[] = array(
				'from'		=> array( "core_tags_cache", "t" ),
				'where'		=> array( "t.tag_cache_key=MD5(CONCAT(?,{$itemClass::$databaseTable}.{$idColumn}))", static::$application . ';' . static::$module . ';' ),
			);			
		}
		
		$permWhere[] = array( 'app=? and perm_type=?', 'videobox', 'category' );
		
		$permArr = \IPS\Member::loggedIn()->permissionArray();
		if ( ($key = array_search('cm', $permArr)) !== false ) {
			unset( $permArr[$key] );
		}		

		$permWhere[] = array( '(' . \IPS\Db::i()->findInSet( 'perm_' . \IPS\videobox\Category::$permissionMap['read'], $permArr ) . ' OR ' . 'perm_' . \IPS\videobox\Category::$permissionMap['read'] . '=? )', '*' );

		foreach( \IPS\Db::i()->select( 'perm_type_id', 'core_permission_index', $permWhere ) as $category )
		{
			$categories[]	= $category;
		}
		if ( \count( $categories ) > 0 )
		{
			$where[] = array( 'v_cat IN (' . implode(',', $categories) . ')' );
		}
		
		return parent::getItemsWithPermission( $where, $order, $limit, $permissionKey, $includeHiddenItems, $queryFlags, $member, $joinContainer, $joinComments, $joinReviews, $countOnly, $joins, $skipPermission, $joinTags, $joinAuthor, $joinLastCommenter, $showMovedLinks );
	}
	
	/**
	 * Should new items be moderated?
	 *
	 * @param	\IPS\Member		$member		The member posting
	 * @param	\IPS\Node\Model	$container	The container
	 * @return	bool
	 */
	public static function moderateNewItems( \IPS\Member $member, \IPS\Node\Model $container = NULL, $considerPostBeforeRegistering = FALSE )
	{
		if ( $container and $container->bitoptions['moderation'] and !$member->group['g_avoid_q'] )
		{
			return TRUE;
		}
		
		return parent::moderateNewItems( $member, $container );
	}

	/**
	 * Get available comment/review tabs
	 *
	 * @return	array
	 */
	public function commentReviewTabs()
	{
		$tabs = array();
		
		if ( $this->container()->bitoptions['reviews'] )
		{
			$tabs['reviews'] = \IPS\Member::loggedIn()->language()->addToStack( 'vb_review_count', TRUE, array( 'pluralize' => array( $this->mapped('num_reviews') ) ) );
		}
		if ( $this->container()->bitoptions['comments'] )
		{
			$tabs['comments'] = \IPS\Member::loggedIn()->language()->addToStack( 'vb_comment_count', TRUE, array( 'pluralize' => array( $this->mapped('num_comments') ) ) );
		}

		return $tabs;
	}
	
	/**
	 * Get comment/review output
	 *
	 * @param	string	$tab	Active tab
	 * @return	string
	 */
	public function commentReviews( $tab )
	{
		if ( $tab === 'reviews' )
		{
			return \IPS\Theme::i()->getTemplate('view')->reviews( $this );
		}
		elseif( $tab === 'comments' )
		{
			return \IPS\Theme::i()->getTemplate('view')->comments( $this );
		}
		
		return '';
	}
	
	/**
	 * Should new comments be moderated?
	 *
	 * @param	\IPS\Member	$member	The member posting
	 * @return	bool
	 */
	public function moderateNewComments( \IPS\Member $member, $considerPostBeforeRegistering = FALSE )
	{
		$commentClass = static::$commentClass;
		return ( $this->container()->bitoptions['comment_moderation'] and !$member->group['g_avoid_q'] ) or parent::moderateNewComments( $member );
	}
			
	/**
	 * Should new reviews be moderated?
	 *
	 * @param	\IPS\Member	$member	The member posting
	 * @return	bool
	 */
	public function moderateNewReviews( \IPS\Member $member, $considerPostBeforeRegistering = FALSE )
	{
		$reviewClass = static::$reviewClass;
		return $this->container()->bitoptions['review_moderation'] and !$member->group['g_avoid_q'];
	}
	
	/**
	 * Can review?
	 *
	 * @param	\IPS\Member\NULL	$member	The member (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canReview($member = NULL, $considerPostBeforeRegistering = true)
	{
		return parent::canReview( $member );
	}	
	
	/**
	 * Get elements for add/edit form
	 *
	 * @param	\IPS\Content\Item|NULL	$item		The current item if editing or NULL if creating
	 * @param	int						$container	Container (e.g. forum) ID, if appropriate
	 * @return	array
	 */
	public static function formElements( $item=NULL, \IPS\Node\Model $container=NULL )
	{
		$return = parent::formElements( $item, $container );
		
		/* Video title */
		$return['title'] = new \IPS\Helpers\Form\Text( 'videobox_title', $item ? $item->title : "", TRUE, array( 'maxLength' => 255 ) );
		
		/* Live Streams */
		if ( \IPS\Settings::i()->vb_conf_lv_on )
		{
			$lvOffothers = array( 'videobox_collections', 'videobox_server_external', 'videobox_server_internal' );
			
			if ( \IPS\Settings::i()->vb_conf_lv_allow_thumb != 1 )
			{
				$lvOffothers = array_merge( array( 'videobox_thumbnail', 'videobox_featuredIMG_original' ), $lvOffothers );
			}
			
			foreach ( (array) $container->cfields as $fv )
			{
				$field_id[] = "videobox_field_" . $fv->id;
			}
			
			if ( \count( $field_id ) > 0 )
			{
				$lvOffothers = array_merge($field_id, $lvOffothers);
			}
			
			$return['livestream_is'] = new \IPS\Helpers\Form\YesNo( 'videobox_livestream_is', $item ? $item->livestream_is : $forceLV, FALSE, array( 'togglesOff' => $lvOffothers, 'togglesOn' => array( 'videobox_livestream' ) ) );
			
			$return['livestream'] = new \IPS\Helpers\Form\Url( 'videobox_livestream', $item ? $item->livestream : "", FALSE, array(), NULL, NULL, NULL, "videobox_livestream" );		
		}
			
		/* Thumbnail */
		$return['thumbnail'] = new \IPS\Helpers\Form\Upload( 'videobox_thumbnail', ( $item AND $item->thumbnail ) ? \IPS\File::get( 'videobox_IMG', $item->thumbnail ) : NULL, FALSE, array( 
			'storageExtension' 	=> 'videobox_IMG', 
			'image' 			=> TRUE,
			'maxFileSize'		=> \IPS\Settings::i()->vb_conf_thumbnail_uploadsize ? \IPS\Settings::i()->vb_conf_thumbnail_uploadsize : 2, 
		), NULL, NULL, NULL, "videobox_thumbnail" );	
		
		/* Featured videos */
		if ( \IPS\Settings::i()->vb_conf_featured_whocanupload && \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_featured_whocanupload ) ) )
		{
			$return['featuredIMG_original'] = new \IPS\Helpers\Form\Upload( 'videobox_featuredIMG_original', ( $item AND $item->featuredIMG_original ) ? \IPS\File::get( 'videobox_Slider', $item->featuredIMG_original ) : NULL, FALSE, array( 
				'storageExtension' 	=> 'videobox_Slider', 
				'image' 			=> TRUE,
				'maxFileSize'		=> \IPS\Settings::i()->vb_conf_thumbnail_uploadsize ? \IPS\Settings::i()->vb_conf_thumbnail_uploadsize : 2, 
			), NULL, NULL, NULL, "videobox_featuredIMG_original" );	
		}
		
		/* Video Content */
		if ( $container->bitoptions['offDescription'] != 1 )
		{		
			$return['content']	= new \IPS\Helpers\Form\Editor( 'videobox_content', $item ? $item->content : NULL, FALSE, array( 'app' => 'videobox', 'key' => 'Video', 'autoSaveKey' => 'videobox-new-video', 'attachIds' => ( $item === NULL ? NULL : array( $item->id, NULL, 'desc' ) ) ), NULL, NULL, NULL, "videobox_content" );
		}
		
		/* Nexus Integration */
		if( \IPS\Application::appIsEnabled( 'nexus' ) and $container->nexus_on and \IPS\Member::loggedIn()->inGroup( explode(",", $container->nexus_can_add) ) )
		{
			$options = array(
				'free'		=> 'video_free',
				'paid'		=> 'video_paid',
			);

			$return['video_cost_type'] = new \IPS\Helpers\Form\Radio( 'video_cost_type', $item && $item->cost ? 'paid' : 'free', TRUE, array(
				'options'	=> $options,
				'toggles'	=> array( 'paid' => array( 'video_cost' ) )
			) );
			
			$commissionBlurb = NULL;
			$fees = NULL;
			if ( $_fees = json_decode( $container->nexus_transfee, TRUE ) )
			{
				$fees = array();
				foreach ( $_fees as $fee )
				{
					$fees[] = (string) ( new \IPS\nexus\Money( $fee['amount'], $fee['currency'] ) );
				}
				$fees = \IPS\Member::loggedIn()->language()->formatList( $fees, \IPS\Member::loggedIn()->language()->get('or_list_format') );
			}
			if ( $container->nexus_percent and $fees )
			{
				$commissionBlurb = \IPS\Member::loggedIn()->language()->addToStack( 'video_cost_desc_both', FALSE, array( 'sprintf' => array( $container->nexus_percent, $fees ) ) );
			}
			elseif ( $container->nexus_percent )
			{
				$commissionBlurb = \IPS\Member::loggedIn()->language()->addToStack('video_cost_desc_percent', FALSE, array( 'sprintf' => $container->nexus_percent ) );
			}
			elseif ( $fees )
			{
				$commissionBlurb = \IPS\Member::loggedIn()->language()->addToStack('video_cost_desc_fee', FALSE, array( 'sprintf' => $fees ) );
			}
			
			\IPS\Member::loggedIn()->language()->words['video_cost_desc'] = $commissionBlurb;			
			$return['video_cost'] = new \IPS\nexus\Form\Money( 'video_cost', $item ? json_decode( $item->cost, TRUE ) : array(), NULL, array(), NULL, NULL, NULL, 'video_cost' );		
		}		
			
		/* Custom Fields */
		$customFieldValues = $item ? $item->customFields() : array();		
		foreach ( (array) $container->cfields as $k => $field )
		{
			$return['videobox_field_' . $k] = $field->buildHelper( isset( $customFieldValues[ "field_{$k}" ] ) ? $customFieldValues[ "field_{$k}" ] : NULL );
		}
		
		/* Build the topic state toggles */
		$options = array();
		$toggles = array();

		if ( static::modPermission( 'lock', NULL, $container ) )
		{
			$options['lock'] = 'videobox_post_locked';
			$toggles['lock'] = array( 'videobox_post_locked' );
			if ( $item AND $item->locked === 1 )
			{
				$values[] = 'lock';
			}			
		}
		
		if ( static::modPermission( 'pin', NULL, $container ) )
		{
			$options['pin'] = 'videobox_post_pinned';
			$toggles['pin'] = array( 'videobox_post_pinned' );
			if ( $item AND $item->pinned )
			{
				$values[] = 'pin';
			}			
		}
		
		if ( static::modPermission( 'hide', NULL, $container ) )
		{
			$options['hide'] = 'videobox_post_hidden';
			$toggles['hide'] = array( 'videobox_post_hidden' );
			if ( $item AND $item->open != 1 )
			{
				$values[] = 'hide';
			}			
		}
		
		if ( static::modPermission( 'feature', NULL, $container ) )
		{
			$options['feature'] = 'videobox_post_featured';
			$toggles['feature'] = array( 'videobox_post_featured' );
			if ( $item AND $item->featured === 1 )
			{
				$values[] = 'feature';
			}			
		}

		if ( \count( $options ) or \count( $toggles ) )
		{
			$return['video_afterPosting'] = new \IPS\Helpers\Form\CheckboxSet( 'video_afterPosting', $values ? $values : array(), FALSE, array(
				'options' 	=> $options,
				'toggles'	=> $toggles,
				'multiple'	=> TRUE
			) );	
		}		
		
		/* Collections */
		$return['video_addToCollection'] = new \IPS\Helpers\Form\Node( 'video_addToCollection', $item->collections ? explode(',', $item->collections) : NULL, FALSE, array(
				'class'					=> 'IPS\videobox\Collections\Collection',
				'multiple'				=> TRUE,
		) );
			
		/* Update submitted time */
		if( $item )
		{
			$return['video_updateTime'] = new \IPS\Helpers\Form\YesNo( 'video_updateTime', FALSE, FALSE );
		}

		/* Video Parts */
		if ( \IPS\Settings::i()->vb_conf_online_on == 1 && \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_post ) ) && ( \IPS\Settings::i()->vb_conf_online_cats=='0' || \in_array( $container->id, explode(",",\IPS\Settings::i()->vb_conf_online_cats) ) ) )
		{
			if ( $item )
			{
				$data = $item->videoOnline();
			}
			else
			{
				foreach( \IPS\Db::i()->select( '*', 'videobox_servers', NULL, 'vbs_position ASC' ) as $row )
				{
					$data[ $row['vbs_name'] ]['id'] = $row['vbs_id'];			
					$data[ $row['vbs_name'] ]['upload'] = $row['vbs_upload'];	
					$data[ $row['vbs_name'] ]['required'] = $row['vbs_not_null'];	
				}
			}

			if ( \count( $data ) > 0 )
			{
				foreach ( $data as $name => $server )
				{
					\IPS\Member::loggedIn()->language()->words[ 'server_' . $server['id'] ] = $name; 
					$required = $server['required'] == 1 ? TRUE : FALSE;

					if ( $server['upload'] != 1 )
					{
						$return[ 'videoOnline_' . $server['id'] ] = new \IPS\Helpers\Form\Custom( 'server_' . $server['id'], NULL, $required, array( 
							'getHtml' => function( $element ) use ( $name, $server )
							{
								return \IPS\Theme::i()->getTemplate( 'post', 'videobox', 'front' )->postVideos($name, $server);
							},
							'validate'	=> function( $element ) use ( $name, $server )
							{
								if ( $element->required )
								{
									$serverValues = array_filter($element->value);
									if ( \count( $serverValues ) < 1 )
									{
										throw new \InvalidArgumentException('form_required');
									}
								}
							}
						), NULL, NULL, NULL, 'videobox_server_external' );
						
					}
					else
					{
						foreach( \IPS\Db::i()->select( '*', 'videobox_online', array( "vbo_video=? and vbo_upload=1", $item->id ), 'vbo_id ASC' ) as $row )
						{
							$uploadedVideos[] = \IPS\File::get( 'videobox_Videos', $row['vbo_url'] );
						}
									
						$return[ 'videoOnline_' . $server['id'] ] = new \IPS\Helpers\Form\Upload( 'server_' . $server['id'], $uploadedVideos, $required, array( 
							'storageExtension' 	=> 'videobox_Subtitles', 
							'allowedFileTypes'	=> array( 'mp4', 'flv', 'm4v', 'webm', 'mp3' ), 
							'maxFileSize'		=> \IPS\Settings::i()->vb_conf_videouploadsize ? \IPS\Settings::i()->vb_conf_videouploadsize : 10, 
							'template' 			=> 'core.attachments.fileItem',
							'multiple'			=> TRUE 
						), NULL, NULL, NULL, 'videobox_server_internal' );
					}
				}	
			}
		}
		
		/* Done */
		return $return;
	}	

	/**
	 * Build form to create
	 *
	 * @param	\IPS\Node\Model|NULL	$container	Container (e.g. forum), if appropriate
	 * @return	\IPS\Helpers\Form
	 */

	protected static function buildCreateForm( \IPS\Node\Model $container=NULL, \IPS\Content\Item $item=NULL )
	{
		$form = new \IPS\Helpers\Form( 'form', \IPS\Member::loggedIn()->language()->checkKeyExists( static::$formLangPrefix . '_save' ) ? static::$formLangPrefix . '_save' : 'save' );
		$form->class = 'ipsForm_vertical';
		$formElements = static::formElements( NULL, $container );

		$form->addTab( static::$formLangPrefix . 'mainTab' );

		foreach ( $formElements as $key => $object )
		{
			if ( \is_object( $object ) )
			{
				$form->add( $object );
			}
			else
			{
				$form->addMessage( $object, NULL, FALSE, $key );
			}
		}
		
		return $form;
	}	
	
	/**
	 * Process create/edit form
	 *
	 * @param	array				$values	Values from form
	 * @return	void
	 */
	public function processForm( $values )
	{		
		parent::processForm( $values );

		/* Content */
		if ( $this->container()->bitoptions['offDescription'] != 1 )
		{
			$this->content = $values['videobox_content'];
		}
		
		/* Live Stream */
		$this->livestream_is = $values['videobox_livestream_is'];
		$this->livestream = $values['videobox_livestream'];
		$this->livestream_updated = 0;
		
		/* Images Upload */
		if ( $values['videobox_thumbnail'] )
		{
			$this->thumbnail = (string) $values['videobox_thumbnail'];
			$this->thumbnail_medium = \IPS\Application::load('videobox')->createThumb( \IPS\Settings::i()->vb_conf_thumbnail_medium, $values['videobox_thumbnail'], $this->thumbnail_medium, 'videobox_IMG' );
		}
		else
		{
			if ( $this->thumbnail )
			{
				try
				{
					\IPS\File::get( 'videobox_IMG', $this->thumbnail )->delete();
					\IPS\File::get( 'videobox_IMG', $this->thumbnail_medium )->delete();
				}
				catch( \Exception $e ){}
				$this->thumbnail = null;
				$this->thumbnail_medium = null;
			}
		}
		
		/* featuredIMG upload */
		if ( $values['videobox_featuredIMG_original'] )
		{
			$this->featuredIMG_original = (string) $values['videobox_featuredIMG_original'];			
			$this->featuredIMG = \IPS\Application::load('videobox')->createThumb( \IPS\Settings::i()->vb_conf_featured_size, $values['videobox_featuredIMG_original'], $this->featuredIMG, 'videobox_Slider' );
		}
		else
		{
			if ( $this->featuredIMG_original )
			{
				try
				{
					\IPS\File::get( 'videobox_Slider', $this->featuredIMG_original )->delete();
					\IPS\File::get( 'videobox_Slider', $this->featuredIMG )->delete();
				}
				catch( \Exception $e ){}
				$this->featuredIMG_original = null;
				$this->featuredIMG = null;
			}
		}
		
		/* Moderator actions */
		if ( isset( $values['video_afterPosting'] ) )
		{
			$this->locked = \in_array( 'lock', $values['video_afterPosting'] ) ? 1 : 0;
			$this->open = \in_array( 'hide', $values['video_afterPosting'] ) ? 0 : 1;
			$this->pinned = \in_array( 'pin', $values['video_afterPosting'] ) ? 1 : 0;
			$this->featured = \in_array( 'feature', $values['video_afterPosting'] ) ? 1 : 0;
		}	

		/* Collections */
		if ( $values['video_addToCollection'] != "" && \count( $values['video_addToCollection'] ) > 0 )
		{
			$this->collections = implode( ",", array_keys( $values['video_addToCollection'] ) );
		}
		else
		{
			$this->collections = NULL;
		}
		
		if ( $values['video_updateTime'] == 1 )
		{
			$this->updated = time();
		}
		
		/* Save */
		$this->save();
		
		/* Save paid video */
		if( \IPS\Application::appIsEnabled( 'nexus' ) and $this->container()->nexus_on and \IPS\Member::loggedIn()->inGroup( explode(",", $this->container()->nexus_can_add) ) )
		{
			$this->cost = $values['video_cost_type'] == 'paid' ? json_encode( $values['video_cost'] ) : NULL;
		}
			
		/* Save extra fields */
		$cfields = array();
		foreach ( $this->container()->cfields as $field )
		{
			$helper							 = $field->buildHelper();
			$cfields[ "field_{$field->id}" ] = $helper::stringValue( $values[ "videobox_field_{$field->id}" ] );
			
			if ( $helper instanceof \IPS\Helpers\Form\Editor )
			{
				$field->claimAttachments( $this->id );
			}			
		}

		if ( !empty( $cfields ) )
		{
			\IPS\Db::i()->insert( 'videobox_ccontent', array_merge( array( 'video_id' => $this->id, 'updated' => time() ), $cfields ), TRUE );
		}	
		
		/* Save videos */
		if ( $values['videobox_livestream_is'] != 1 && \IPS\Settings::i()->vb_conf_online_on == 1 && \IPS\Member::loggedIn()->inGroup( explode( ",", \IPS\Settings::i()->vb_conf_online_post ) ) && ( \IPS\Settings::i()->vb_conf_online_cats=='0' || \in_array( $this->container()->id, explode(",",\IPS\Settings::i()->vb_conf_online_cats) ) ) )
		{		
			foreach( \IPS\Db::i()->select( '*', 'videobox_servers', NULL, 'vbs_position ASC' ) as $row )
			{
				if ( $row['vbs_upload'] != 1 )
				{
					$this->saveExternalVideos( $row['vbs_id'] );
				}
				else
				{
					$this->saveUploadVideos( $row['vbs_id'], $values );	
				}
			}
		}
	}		
	
	/*-------------------------------------------------------------------------*/
	// Save video parts
	/*-------------------------------------------------------------------------*/
	public function saveExternalVideos( $svid )
	{	
		# Check server
		try
		{
			$server	= \IPS\videobox\Servers\Server::load( $svid );
		}
		catch( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'videobox_server_not_found', '2BIMVB111/1', 404 );
		}		
				
		# Remove first
		\IPS\Db::i()->delete( 'videobox_online', array( 'vbo_video=? AND vbo_server=?', $this->id, $svid ) );
		
		# Save
		$sv = "server_" . $svid;		
		$svarray = \IPS\Request::i()->$sv;
		
		$sub = "subtitle_" . $svid;
		$subarray = \IPS\Request::i()->$sub;
		
		foreach( (array) $svarray as $k => $row )
		{
			if ( $row )
			{
				$data[$row] = $subarray[$k];
			}
		}

		if ( \count( $data  ) > 0 )
		{
			foreach( $data as $url => $sub )
			{
				$cnt++;
				\IPS\Db::i()->insert( 'videobox_online', array(
					'vbo_video'		=> $this->id,
					'vbo_url'		=> $url,
					'vbo_sub'		=> $sub,					
					'vbo_server'	=> $svid,
				) );
				# Save thumbnails for youtube, vimeo
				if ( $cnt == \count( $data ) )
				{
					$img = \IPS\Application::load('videobox')->getthumbnail($url, true);
					$this->externalThumb = $img['small'];
					$this->externalBigThumb = $img['large'];
					$this->save();
				}			
			}
		}	
	}

	protected function saveUploadVideos( $svid, $values )
	{
		if ( !$svid || !$values )
		{
			return;
		}
		
		$existing = iterator_to_array( \IPS\Db::i()->select( '*', 'videobox_online', array( "vbo_video=? and vbo_upload=1", $this->id ) )->setKeyField( 'vbo_url' ) );

		foreach ( (array) $values[ 'server_' . $svid ] as $file )
		{
			if ( !isset( $existing[ (string) $file ] ) && (string) $file )
			{
				\IPS\Db::i()->insert( 'videobox_online', array(
					'vbo_video'		=> $this->id,
					'vbo_url'		=> (string) $file,
					'vbo_server'	=> $svid,	
					'vbo_upload'	=> 1,
				) );
			}
			unset( $existing[ (string) $file ] );
		}

		if ( \count( $existing ) > 0 )
		{
			foreach ( $existing as $location => $file )
			{
				try
				{
					\IPS\File::get( 'videobox_Videos', $location )->delete();
				}
				catch ( \Exception $e ) {}
				\IPS\Db::i()->delete( 'videobox_online', array( 'vbo_id=?', $file['vbo_id'] ) );
			}
		}
	}

	/* !Tags */
	
	/**
	 * Can tag?
	 *
	 * @param	\IPS\Member|NULL		$member		The member to check for (NULL for currently logged in member)
	 * @param	\IPS\Node\Model|NULL	$container	The container to check if tags can be used in, if applicable
	 * @return	bool
	 */
	public static function canTag( \IPS\Member $member = NULL, \IPS\Node\Model $container = NULL )
	{
		return parent::canTag( $member, $container ) and ( $container === NULL or !$container->tags_disabled );
	}
	
	/**
	 * Can use prefixes?
	 *
	 * @param	\IPS\Member|NULL		$member		The member to check for (NULL for currently logged in member)
	 * @param	\IPS\Node\Model|NULL	$container	The container to check if tags can be used in, if applicable
	 * @return	bool
	 */
	public static function canPrefix( \IPS\Member $member = NULL, \IPS\Node\Model $container = NULL )
	{
		return parent::canPrefix( $member, $container ) and ( $container === NULL or !$container->tags_noprefixes );
	}
	
	/**
	 * Defined Tags
	 *
	 * @param	\IPS\Node\Model|NULL	$container	The container to check if tags can be used in, if applicable
	 * @return	array
	 */
	public static function definedTags( \IPS\Node\Model $container = NULL )
	{
		if ( $container and $container->tags_predefined )
		{
			return explode( ',', $container->tags_predefined );
		}
		
		return parent::definedTags( $container );
	}
	
	/**
	 * Get custom field values
	 * @return	array
	 */
	public function customFields()
	{
		$return = array();
		$fields = $this->container()->cfields;

		try
		{
			$data = \IPS\Db::i()->select( '*', 'videobox_ccontent', array( 'video_id=?', $this->id ) )->first();
			
			foreach ( $data as $k => $v )
			{
				if ( array_key_exists( str_replace( 'field_', '', $k ), $fields ) )
				{
					$return[ $k ] = $v;
				}
			}
		}
		catch( \UnderflowException $e ){}
		
		return $return;
	}
	
	/**
	 * Defined Custom Fields
	 * @return	array
	 */	
	public function cfields()
	{
		$cfields	= array();
		$fields		= $this->customFields();

		foreach( $this->container()->cfields AS $k => $field )
		{
			if ( \in_array( $k, explode(",", $this->container()->hidefields ) ) )
			{
				continue;
			}
			if ( isset( $fields['field_' . $k ] ) )
			{	
				$fvalue = $field->displayValue( $fields[ 'field_' . $k ] );

				if ( $field->type == 'Upload' ) 
				{
					$val_ = $fields[ 'field_' . $k ];

					$extension = mb_strrchr($val_, ".");

					if ( !\in_array( mb_strtolower($extension), array('.png', '.jpg', '.jpeg', '.png', '.gif', '.bmp', '.tiff', '.webp', '.wbmp', '.swf', '.swc' ) ) )
					{
						$fvalue = (string) \IPS\Http\Url::internal( 'app=videobox&module=video&controller=view&id=' . $this->id . '&do=downloadff&fid=' . $field->id );
					}
				}
				
				if ($field->type == 'Select')
				{
					$fvalue = str_replace( "<br>", ", ", $field->displayValue( $fields[ 'field_' . $k ] ) );		
					$fvalue = strip_tags( $fvalue );
				}	
				
				if ($field->type == 'Url')
				{
					$fvalue = strip_tags($fvalue);
				}
				
				$fvalue = str_replace( "<wbr>", "", $fvalue );
				
				if ( $fvalue )
				{
					$cfields[] = array( 'key'		=> $k,
										'value'		=> $this->viewProtectField($field->id) ? $fvalue : \IPS\Theme::i()->getTemplate( 'view', 'videobox', 'front' )->hiddenField($this),
										'format'	=> $this->viewProtectField($field->id) ? $field->format : ""
					);
				}				
			}
		}
		return $cfields;
	}
	
	/**
	 * viewProtectField
	 *
	 * @return	true/false
	 */	
	protected function viewProtectField( $fieldID )
	{
		if ( !\in_array( $fieldID, explode(",", $this->container()->protectField ) ) )
		{
			return true;
		}
		if ( $this->container()->viewprotectField == 'logintoview' )
		{
			return \IPS\Member::loggedIn()->member_id ? true : false;
		}
		elseif ( $this->container()->viewprotectField == 'liketoview' )
		{
			return ( \IPS\Member::loggedIn()->member_id == $this->author or $this->reacted() ) ? true : false;			
		}
		else
		{
			return \IPS\Member::loggedIn()->member_id && $this->reacted() ? true : false;
		}
	}

	/**
	 * Move
	 *
	 * @param	\IPS\Node\Model	$container	Container to move to
	 * @param	bool			$keepLink	If TRUE, will keep a link in the source
	 * @return	void
	 */
	public function move( \IPS\Node\Model $container, $keepLink=FALSE )
	{
		if(	$container->bitoptions['containerMode'] == 1 )
		{
			throw new \InvalidArgumentException;
		}
		
		parent::move( $container, $keepLink );
	}
	
	/**
	 * Get thumbnail
	 *
	 * @return	array
	 */
	public function thumbnail( $size = NULL )
	{
		if ( $this->livestream_is == 1 )
		{
			switch ( $size )
			{
				default:
					$thumb = $this->lstreamData()->thumbnail; 
					break;
				case 'medium':
					$thumb = $this->lstreamData()->thumbnail; 
					break;
				case 'featured':
					$thumb = $this->lstreamData()->banner; 
					break;
			}
			if ( !$thumb )
			{
				return (string) \IPS\Theme::i()->resource( 'livestream.png', 'videobox', 'front' );
			}
			
			return mb_strpos( $this->livestream, 'youtube.com/channel/' ) === false ? $thumb . "?v=" . time() : $thumb;
		}
		else
		{
			switch ( $size )
			{
				default:
					$thumb = $this->thumbnail ? (string) \IPS\File::get( 'videobox_IMG', $this->thumbnail )->url : $this->externalThumb; 
					break;
				case 'medium':
					$thumb = $this->thumbnail_medium ? (string) \IPS\File::get( 'videobox_IMG', $this->thumbnail_medium )->url : $this->externalThumb; 
					break;
				case 'featured':
					$thumb = $this->featuredIMG ? (string) \IPS\File::get( 'videobox_Slider', $this->featuredIMG )->url : $this->externalBigThumb; 
					break;
			}
			
			if ( ! $thumb )
			{
				$thumbFromContent = \IPS\Application::load('videobox')->getthumbnail( $this->content, true );

				if ( $thumbFromContent )
				{
					$this->externalThumb = $thumbFromContent['small'];
					$this->externalBigThumb = $thumbFromContent['large'];
					$this->save();
					
					return $size == 'featured' ? $thumbFromContent['large'] : $thumbFromContent['small'];
				}
				
				$thumb = (string) \IPS\Theme::i()->resource( 'videoNoThumb.jpg', 'videobox', 'front' );
			}
			
			return $thumb;
		}
	}
	
	/**
	 * Returns the content images
	 *
	 * @param	int|null	$limit	Number of attachments to fetch, or NULL for all
	 *
	 * @return	array|NULL	If array, then array( 'core_Attachment' => 'month_x/foo.gif', ... );
	 * @throws	\BadMethodCallException
	 */
	public function contentImages( $limit = NULL )
	{
		$images = array();
		
		if ( $this->livestream_is != 1 )
		{
			if ( $this->externalThumb )
			{
			//	$images[] = array( 'videobox_IMG' => $this->externalThumb );
			}
			else
			{
				$images[] = array( 'videobox_IMG' => (string) \IPS\File::get( 'videobox_IMG', $this->thumbnail ) );
			}
		}
		else
		{
		//	$images[] = array( 'videobox_IMG' => (string) $this->lstreamData()->thumbnail );
		}

		return \count( $images ) ? $images : NULL;
	}
	
	/*-------------------------------------------------------------------------*/
	// videoOnline
	/*-------------------------------------------------------------------------*/
	protected $_videoOnline = NULL;
	
	public function videoOnline()
	{	
		if ( $this->_videoOnline !== NULL )
		{
			return $this->_videoOnline;
		}
	
		$data = array();
		
		foreach( \IPS\Db::i()->select( '*', 'videobox_servers', NULL, 'vbs_position ASC' )->join( 'videobox_online', 'videobox_servers.vbs_id=videobox_online.vbo_server AND videobox_online.vbo_video=' . $this->id ) as $row )
		{
			$cnt++;
			$data[ $row['vbs_name'] ]['id'] = $row['vbs_id'];			
			$data[ $row['vbs_name'] ]['groups'] = $row['vbs_groups'];
			$data[ $row['vbs_name'] ]['upload'] = $row['vbs_upload'];	
			$data[ $row['vbs_name'] ]['required'] = $row['vbs_not_null'];
			if ( $row['vbo_url'] )
			{
				$data[ $row['vbs_name'] ]['online'][ $row['vbo_id'] ] = $row;
			}
		}

		$this->_videoOnline = $data;
		
		return $this->_videoOnline;
	}	
		
	/*-------------------------------------------------------------------------*/
	// getSubtitles
	/*-------------------------------------------------------------------------*/
	protected $_subtitles = NULL;
	
	public function getSubtitles($txt)
	{
		if ( $this->_subtitles !== NULL )
		{
			return $this->_subtitles;
		}		
		
		$subs = explode( "\n", str_replace( "\r", '', $txt ) );
		$subtitles = array();
		foreach( $subs as $sub )
		{
			$cnt++;
			$v = explode( "|", $sub );
			
			$subsrc = $this->querySub($v[0]);
			
			if ( $subsrc != "")
			{
				$srclang = $v[1];
				$label = \IPS\Application::load('videobox')->searchCountry($srclang);
				$subtitles[] = array( 'sub' => $subsrc, 'lang' => $srclang, 'label' => $label, 'default' => $cnt == 1 ? 1 : 0 );
			}
		}
		
		$this->_subtitles = $subtitles;
		
		return $this->_subtitles;
	}
	
	protected function querySub( $id )
	{
		try
		{
			$data = \IPS\Db::i()->select( '*', 'videobox_subtitles', array( 'vbsub_id=? and vbsub_video=?', $id, $this->id ) )->first();
		}
		catch( \UnderflowException $e ){}
		
		if ( $data )
		{
			return (string) \IPS\File::get( 'videobox_subtitles', $data['vbsub_url'] )->url;				
		}
		else return "";
	}
		
	
	//-----------------------------
	// Video player
	//-----------------------------
	public function player()
	{
		if ( $this->livestream_is == 1 )
		{
			// Delay time to save api quota
			if ( ! \IPS\IN_DEV )
			{
				if ( \mb_strpos( $this->livestream, 'youtube.com/channel/' ) !== false )
				{
					if ( ( time() - $this->livestream_updated ) < 120 && $this->livestream_data && $this->livestream_online == 'online' )
					{
						return \IPS\Theme::i()->getTemplate( 'embed', 'videobox', 'front' )->livestreams( $this, json_decode( $this->livestream_data, true ) );
					}
				}
			}

			$data = \IPS\Application::load('videobox')->lstreamInfo( $this, true );
			return \IPS\Theme::i()->getTemplate( 'embed', 'videobox', 'front' )->livestreams( $this, $data );
		}
		else
		{
			$videoID = \IPS\Request::i()->vid > 0 ? \IPS\Request::i()->vid : 0;

			$cur = array();
			$subs = array();
			$next = array();			
			
			# Get the current & next video
			if ( \IPS\Request::i()->serverID > 0 )
			{
				foreach ( \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_video=? and vbo_server=?', $this->id, (int) \IPS\Request::i()->serverID ), 'vbo_id ASC' )->join('videobox_servers', "videobox_servers.vbs_id=videobox_online.vbo_server" ) as $re )
				{
					$cnt++;
					if ( $re['vbs_groups'] == '0' || \IPS\Member::loggedIn()->inGroup( explode( ",", $re['vbs_groups'] ) ) )
					{
						if ( $videoID == 0 )
						{
							if ( $cnt == 1 )
							{
								$cur = $re;
							}
							elseif ( $cnt == 2 )
							{
								$next = $re; 
								break;
							}
						}
						else
						{
							if ( $re['vbo_id'] == $videoID )
							{
								$cur = $re;
							}
							elseif ( $cur )
							{
								$next = $re; 
								break;
							}							
						}
					}
				}
			}
			else
			{
				foreach ( $this->videoOnline() as $name => $server )
				{
					if ( \count($server['online']) > 0 && ( $server['groups'] == 0 || \IPS\Member::loggedIn()->inGroup( explode( ",", $server['groups'] ) ) ) )
					{
						ksort($server['online']);
						foreach ($server['online'] as $num => $row)
						{
							$cnt++;
							if ( $cnt == 1 )
							{
								$cur = $row;
							}
							elseif ( $cnt == 2 )
							{
								$next = $row;
								break;
							}	
						}		
					}	
				}
			}
			
			if ( $cur['vbs_upload'] == 1 )
			{
				$cur['vbo_url'] = (string) \IPS\File::get( 'videobox_Videos', $cur['vbo_url'] )->url;
			}
		
			return \IPS\Theme::i()->getTemplate( 'embed', 'videobox', 'front' )->player( $this, $cur, $next );
		}
	}
	
	public function myPlayer($url)
	{
		$supportExt = array( ".mp4", ".m4v", ".mp3", ".mov", ".webm", ".m3u8", ".dash" );
		foreach( $supportExt as $a ) 
		{
			if ( mb_stripos( $url, $a ) !== false )
			{
				return true;
			}
		}

		return false;
	}
	
	/*-------------------------------------------------------------------------*/
	// truncateText
	/*-------------------------------------------------------------------------*/
	public function truncateText( $text, $chars=300 )
	{	
		$text = $text." ";
		$text = mb_substr($text, 0, $chars);
		$text = mb_substr($text, 0, mb_strrpos($text,' '));
		$text = $text."...";
		return $text;
	}	
	
	//-----------------------------
	// Get live stream data
	//-----------------------------	
	public function lstreamData()
	{
		$db = array();
		if ( $this->livestream_is == 1 && $this->livestream_data )
		{
			$db = @json_decode($this->livestream_data);
		}
		if ( $db->online == "offline" && $this->thumbnail )
		{
			$db->thumbnail = (string) \IPS\File::get( 'videobox_IMG', $this->thumbnail_medium )->url;
			$db->banner = (string) \IPS\File::get( 'videobox_Slider', $this->featuredIMG )->url;
		}

		return $db;
	}
	
	/**
	 * Reaction type
	 *
	 * @return	string
	 */
	public static function reactionType()
	{
		return 'v_id';
	}
	
	/**
	 * Supported Meta Data Types
	 *
	 * @return	array|NULL
	 */
	public static function supportedMetaDataTypes()
	{
		return array( 'core_FeaturedComments', 'core_ContentMessages' );
	}	
	
	/**
	 * Get image for embed
	 *
	 * @return	\IPS\File|NULL
	 */
	public function embedImage()
	{
		return \IPS\File::get( 'videobox_IMG', $this->thumbnail() );
	}	
	
	/**
	 * Get widget sort options
	 *
	 * @return array
	 */
	public static function getWidgetSortOptions()
	{
		$sortOptions = parent::getWidgetSortOptions();
		
		$sortOptions['livestream_online'] = 'vbox_lstream_status';
		$sortOptions['_rand'] = 'sort_rand';
		
		return $sortOptions;
	}
	
	/**
	 * Is this a paid?
	 *
	 * @return	bool
	 */
	public function isPaid()
	{
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and $this->container()->nexus_on)
		{
			if ( $this->cost )
			{
				$costs = json_decode( $this->cost, TRUE );
				if ( \is_array( $costs ) )
				{
					foreach ( $costs as $currency => $data )
					{
						if ( $data['amount'] )
						{
							return TRUE;
						}
					}
				}
				else
				{
					return TRUE;
				}
			}
		}
		
		return FALSE;
	}	
	
	/**
	 * Get Price
	 *
	 * @return	\IPS\nexus\Money|NULL
	 */
	public function price()
	{
		$cost = $this->cost;
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and $this->container()->nexus_on )
		{
			if ( $cost )
			{
				$currency = ( isset( $_SESSION['currency'] ) and \in_array( $_SESSION['currency'], \IPS\nexus\Money::currencies() ) ) ? $_SESSION['currency'] : \IPS\nexus\Customer::loggedIn()->defaultCurrency();
				if ( $costs = json_decode( $cost, TRUE ) )
				{
					if ( isset( $costs[ $currency ]['amount'] ) and $costs[ $currency ]['amount'] )
					{
						return new \IPS\nexus\Money( $costs[ $currency ]['amount'], $currency );
					}
				}
				else
				{
					return new \IPS\nexus\Money( $cost, $currency );
				}
			}
		}
		
		return NULL;
	}
	
	/**
	 * @brief	Number of purchases
	 */
	protected static $purchaseCounts;
	
	/**
	 * Get number of purchases
	 *
	 * @return	array
	 */
	public function purchaseCount()
	{
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and $this->container()->nexus_on )
		{
			if ( static::$purchaseCounts === NULL )
			{
				static::$purchaseCounts = iterator_to_array( \IPS\Db::i()->select( 'COUNT(*) AS count, ps_item_id', 'nexus_purchases', array( array( 'ps_app=? AND ps_type=?', 'videobox', 'video' ), \IPS\Db::i()->in( 'ps_item_id', array_keys( static::$multitons ) ) ), NULL, NULL, 'ps_item_id' )->setKeyField('ps_item_id')->setValueField('count') );
				foreach ( array_keys( static::$multitons ) as $k )
				{
					if ( !isset( static::$purchaseCounts[ $k ] ) )
					{
						static::$purchaseCounts[ $k ] = 0;
					}
				}
			}
			
			if ( !isset( static::$purchaseCounts[ $this->id ] ) )
			{
				static::$purchaseCounts[ $this->id ] = \IPS\Db::i()->select( 'COUNT(*)', 'nexus_purchases', array( 'ps_app=? AND ps_type=? AND ps_item_id=?', 'videobox', 'video', $this->id ) )->first();
			}

			return static::$purchaseCounts[ $this->id ];
		}
		
		return NULL;
	}
	
	/**
	 * Can the member buy this video?
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	bool
	 */
	public function canBuy( \IPS\Member $member = NULL )
	{
		/* Is this a paid video? */
		if ( !\IPS\Application::appIsEnabled( 'nexus' ) or !$this->container()->nexus_on or !$this->cost or !$this->price() )
		{
			return FALSE;
		}
		
		/* Init */
		$member = $member ?: \IPS\Member::loggedIn();

        /* author */
        if( $member == $this->author() )
        {
            return FALSE;
        }
		
		/* Basic permission check */
		if ( !$this->container()->can( 'read', $member ) )
		{
			/* Hold on - if we're a guest and buying means we'll have to register which will put us in a group with permission, we can continue */
			if ( \IPS\Member::loggedIn()->member_id or !$this->container()->can( 'read', \IPS\Member\Group::load( \IPS\Settings::i()->member_group ) ) )
			{
				return FALSE;
			}
		}

		return TRUE;
	}
		
	# POINTS
    public static $pointsTable;
    public static $pointsMemcolumn;
    public static $pointsColumn;
    	
	public static function checkPointsDB()
	{
		if( ! \IPS\Settings::i()->vb_conf_points_on )
		{
			return false;
		}

		if ( \IPS\Settings::i()->vb_conf_points_table == 'core_members' )
		{
			self::$pointsTable = \IPS\Settings::i()->vb_conf_points_table;
			self::$pointsMemcolumn = "member_id";
			self::$pointsColumn = \IPS\Settings::i()->vb_conf_points_column;

			if( ! \IPS\Db::i()->checkForColumn( self::$pointsTable, self::$pointsColumn ) )
			{
				return false;
			}			
		}
		else
		{
			$_custom = explode('|', \IPS\Settings::i()->vb_conf_points_table );
			
			if( \count( $_custom ) > 1 )
			{
				self::$pointsTable = $_custom[0];
				self::$pointsMemcolumn = $_custom[1];
				self::$pointsColumn = \IPS\Settings::i()->vb_conf_points_column;
				
				if( ! \IPS\Db::i()->checkForTable( self::$pointsTable ) )
				{		
					return false;
				}
		
				if( ! \IPS\Db::i()->checkForColumn( self::$pointsTable, self::$pointsMemcolumn ) || ! \IPS\Db::i()->checkForColumn( self::$pointsTable, self::$pointsColumn ) )
				{
					return false;
				}					
			}				
		}

		return true;
	}
	
	public static function updatePoints( $member_id = 0, $points = 0, $minus = false )
	{
		if( ! \IPS\Settings::i()->vb_conf_points_on || $points == 0 )
		{
			return false;
		}
		
		$currentPoints = \IPS\Db::i()->select( self::$pointsColumn, self::$pointsTable, array( self::$pointsMemcolumn . '=?', $member_id ) )->first();

		if( ! $minus )
		{
			$currentPoints += $points;
		}
		else
		{
			if ( $currentPoints >= $points )
			{
				$currentPoints -= $points;
			}
		}
		
		\IPS\Db::i()->update( self::$pointsTable, array( self::$pointsColumn => $currentPoints ), array( self::$pointsMemcolumn . ' = ?', $member_id ) ); 			
	}
	
	/**
	 * Delete video
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		
		// Delete thumbnail
		if ( $this->thumbnail )
		{
			try
			{			
				\IPS\File::get( 'videobox_IMG', $this->thumbnail )->delete();
				\IPS\File::get( 'videobox_IMG', $this->thumbnail_medium )->delete();
			}
			catch( \Exception $e ){}
		}
		
		if ( $this->featuredIMG )
		{		
			try
			{			
				\IPS\File::get( 'videobox_Slider', $this->featuredIMG )->delete();
				\IPS\File::get( 'videobox_Slider', $this->featuredIMG_original )->delete();
			}
			catch( \Exception $e ){}		
		}
		
		// Delete field content
		\IPS\Db::i()->delete( 'videobox_ccontent', array( 'video_id=?', $this->id ) );
		
		// Delete video online
		\IPS\Db::i()->delete( 'videobox_online', array( 'vbo_video=? AND vbo_upload=0', $this->id ) );		
		foreach( \IPS\Db::i()->select( '*', 'videobox_online', array( 'vbo_video=? AND vbo_upload=1', $this->id ) ) as $onl )
		{
			try
			{
				\IPS\File::get( 'videobox_Videos', $onl['vbo_url'] )->delete();			
			}
			catch( \Exception $e ){}
		}
		\IPS\Db::i()->delete( 'videobox_online', array( 'vbo_video=? AND vbo_upload=1', $this->id ) );
		
		// Delete subtitles
		foreach( \IPS\Db::i()->select( '*', 'videobox_subtitles', array( 'vbsub_video=?', $this->id ) ) as $video )
		{
			try
			{
				\IPS\File::get( 'videobox_Subtitles', $video['vbsub_url'] )->delete();			
			}
			catch( \Exception $e ){}
		}	
		
		// Points
		if ( \IPS\Settings::i()->vb_conf_points_on == 1 && \IPS\Settings::i()->vb_conf_points_decrease == 1 )
		{		
			$newPoints = $this->container()->points_on == 1 ? $this->container()->points_video : \IPS\Settings::i()->vb_conf_points_video;

			if ( $newPoints == -1 )
			{
				return;
			}
						
			if( self::checkPointsDB() )
			{		
				self::updatePoints( $this->author()->member_id, $newPoints, true );
			}				
		}
		
		// Delete topic
		if ( $topic = $this->topic() and $this->container()->topic_delete )
		{
			$topic->delete();
		}		
	}
	
	/**
	 * Get output for API
	 *
	 * @param	\IPS\Member|NULL	$authorizedMember	The member making the API request or NULL for API Key / client_credentials
	 * @param	array	$backup	If provided, will output for a particular version - provide row from downloads_filebackup
	 * @return	array
	 * @apiresponse	int								id					ID number
	 * @apiresponse	string							title				Title
	 * @apiresponse	\IPS\downloads\Category			category			Category
	 * @apiresponse	\IPS\Member						author				Author
	 * @apiresponse	datetime						date				When the video was posted
	 * @apiresponse	int								comments			Number of comments
	 * @apiresponse	int								reviews				Number of reviews
	 * @apiresponse	int								views				Number of views
	 * @apiresponse	string							prefix				The prefix tag, if there is one
	 * @apiresponse	[string]						image				Images
	 * @apiresponse	[array]							tags				The tags
	 * @apiresponse	bool							locked				File is locked
	 * @apiresponse	bool							hidden				File is hidden
	 * @apiresponse	bool							pinned				File is pinned
	 * @apiresponse	bool							featured			File is featured
	 * @apiresponse	string							url					URL
	 * @apiresponse	string							rating				Average Rating
	 */
	public function apiOutput( \IPS\Member $authorizedMember = NULL )
	{
		return array(
			'id'			=> $this->id,
			'title'			=> $this->title,
			'category'		=> $this->container()->apiOutput( $authorizedMember ),
			'date'			=> $this->started,
			'comments'		=> $this->comments,
			'reviews'		=> $this->reviews,
			'views'			=> $this->views,
			'prefix'		=> $this->prefix(),
			'tags'			=> $this->tags(),
			'images'			=> array( 'thumbnail' => $this->thumbnail('medium'), 'image' => $this->thumbnail(), 'featured' => $this->thumbnail('featured') ),
			'videos'		=> $this->videoOnline(),
			'locked'		=> (bool) $this->locked(),
			'hidden'		=> (bool) $this->hidden(),
			'pinned'		=> (bool) $this->mapped('pinned'),
			'featured'		=> (bool) $this->mapped('featured'),
			'url'			=> (string) $this->url(),
			'rating'		=> $this->averageRating(),
		);
	}
	
	/**
	 * Get HTML for search result display
	 *
	 * @return	callable
	 */
	public function approvalQueueHtml( $ref=NULL, $container, $title )
	{
		if ( \IPS\Settings::i()->vb_conf_online_on == 1 && ( \IPS\Settings::i()->vb_conf_online_cats=='0' || \in_array( $container->id, explode(",",\IPS\Settings::i()->vb_conf_online_cats) ) ) )
		{
			$this->videoJS();
		}		
		return \IPS\Theme::i()->getTemplate( 'global', 'videobox', 'front' )->approvalQueueItem( $this, $ref, $container, $title );
	}	
	
	/**
	 * JS and CSS
	 */	
	public function videoJS()
	{
		# CSS
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'video-js.css', 'videobox', 'front' ) );	
		
		# Javascript
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'video-js/video.min.js', 'videobox', 'interface' ) );
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'video-js/plugins/videojs-resolution-switcher.js', 'videobox', 'interface' ) );				
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'video-js/plugins/hls.js', 'videobox', 'interface' ) );				
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'video-js/plugins/ga.js', 'videobox', 'interface' ) );	
		if (\IPS\Settings::i()->vb_conf_online_preroll == 1 )
		{
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'video-js/plugins/ads.js', 'videobox', 'interface' ) );	
		}
	}	
	
	/**
	 * Process created object AFTER the object has been created
	 *
	 * @param	\IPS\Content\Comment|NULL	$comment	The first comment
	 * @param	array						$values		Values from form
	 * @return	void
	 */	
	protected function processAfterCreate( $comment, $values )
	{
		\IPS\File::claimAttachments( 'videobox-new-video', $this->id, NULL, 'desc' );			
		
		if ( \IPS\Settings::i()->vb_conf_points_on == 1 )
		{
			$newPoints = $this->container()->points_on == 1 ? $this->container()->points_video : \IPS\Settings::i()->vb_conf_points_video;

			if ( $newPoints == -1 )
			{
				return;
			}
			
			if( self::checkPointsDB() )
			{		
				self::updatePoints( $this->author()->member_id, $newPoints );
			}		
		}
		
		if ( \IPS\Application::appIsEnabled('forums') and $this->container()->forum_id and !$this->hidden() )
		{
			try
			{
				$this->syncTopic();
			}
			catch( \Exception $ex ) { }
		}
				
		parent::processAfterCreate( $comment, $values );
	}

		
	/**
	 * Process after the object has been edited on the front-end
	 *
	 * @param	array	$values		Values from form
	 * @return	void
	 */
	public function processAfterEdit( $values )
	{
		if ( \IPS\Application::appIsEnabled('forums') and $this->container()->forum_id and !$this->hidden() )
		{
			try
			{
				$this->syncTopic();
			}
			catch( \Exception $ex ) { }
		}

		parent::processAfterEdit( $values );
	}

	/**
	 * Callback to execute when tags are edited
	 *
	 * @return	void
	 */
	protected function processAfterTagUpdate()
	{
		parent::processAfterTagUpdate();

		if ( \IPS\Application::appIsEnabled('forums') and $this->container()->forum_id and !$this->hidden() )
		{
			try
			{
				$this->syncTopic();
			}
			catch( \Exception $ex ) { }
		}
	}
	
	/**
	 * Give a content item the opportunity to filter similar content
	 * 
	 * @note Intentionally blank but can be overridden by child classes
	 * @return array|NULL
	 */
	public function similarContentFilter()
	{
		return array(
			array( 'tag_meta_app=? and tag_meta_area=? and tag_meta_id<>?', 'videobox', 'video', $this->id )
		);
	}
	
	/**
	 * Get the topic title
	 *
	 * @return string
	 */
	function getTopicTitle()
	{
		return $this->container()->topic_prefix . $this->title . $this->container()->topic_suffix;
	}

	/**
	 * Get the topic content
	 *
	 * @return mixed
	 */
	function getTopicContent()
	{
		return \IPS\Theme::i()->getTemplate( 'post', 'videobox', 'front' )->topic( $this );
	}	


	/**
	 * Process the review form
	 *
	 * @param	array	$values		Array of $form values
	 * @return  \IPS\Content\Comment
	 */
	public function processReviewForm( $values )
	{
		$this->_averageReviewRating = NULL; // sua loi ko cap nhat rating :( ko hieu vi sao luon
		return parent::processReviewForm( $values );
	}
	
	/**
	 * Get template for content tables
	 *
	 * @return	callable
	 */
	public static function contentTableTemplate()
	{
		return array( \IPS\Theme::i()->getTemplate( 'browse', 'videobox', 'front' ), 'categoryRows_grid' );
	}	
	

	
	
	
	
	
}