<?php
/**
 * @brief		LatestReviews Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	videobox
 * @since		12 Aug 2017
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * LatestReviews Widget
 */
class _LatestReviews extends \IPS\Content\WidgetComment
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'LatestReviews';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Class
	 */
	protected static $class = 'IPS\videobox\Video\Review';
	
	/**
	 * @brief	Moderator permission to generate caches on [optional]
	 */
	protected $moderatorPermissions	= array( 'can_view_hidden_content', 'can_view_hidden_videobox_video_review' );
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		$form = parent::configuration( $form );
		
 		$form->add( new \IPS\Helpers\Form\YesNo( 'vbox_block_reviews_filled', $this->configuration['vbox_block_reviews_filled'], FALSE ) );
		
		return $form;
 	} 
 	
	/**
	 * Get where clause
	 *
	 * @return	array
	 */
	protected function buildWhere()
	{
		$where = parent::buildWhere();
		if ( $this->configuration['vbox_block_reviews_filled'] == 1 )
		{
			$where[] = array( "vbrv_text != '' AND vbrv_text IS NOT NULL" );
		}
		return $where;
	}
	
}