<?php
/**
 * @brief		TopPosters Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		04 Jan 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * topPoster Widget
 */
class _TopPosters extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'TopPosters';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
		// Use this to perform any set up and to assign a template that is not in the following format:
		// $this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
 		{
	 		$form = new \IPS\Helpers\Form;
 		} 
 		
		$form->add( new \IPS\Helpers\Form\Number( 'number_to_show', isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 5, TRUE ) );
		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		foreach ( array( 'week' => 'P1W', 'month' => 'P1M', 'year' => 'P1Y', 'all' => NULL ) as $time => $interval )
		{
			/* What's the time period we care about? */
			$intervalWhere = NULL;
			if ( $interval )
			{
				$intervalWhere = array( 'v_started>?', \IPS\DateTime::create()->sub( new \DateInterval( $interval ) )->getTimestamp() );
			}
			
			/* Get the submitters ordered by their rating */
			$where = array( array( 'v_author != ? AND v_open = 1', 0 ) );
			if ( $interval )
			{
				$where[] = $intervalWhere;
			}
			$ratings = iterator_to_array( \IPS\Db::i()->select(
				'videobox_videos.v_author, AVG(v_rating) as rating, COUNT(v_id) AS videos',
				'videobox_videos',
				$where,
				'videos DESC, rating DESC',
				isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 5, 'v_author'
			)->setKeyField('v_author')->setValueField('rating') );
			
			/* Get their video counts */
			$where = array( array( \IPS\Db::i()->in( 'v_author', array_keys( $ratings ) ) ) );
			if ( $interval )
			{
				$where[] = $intervalWhere;
			}
			
			$videoCounts = iterator_to_array( \IPS\Db::i()->select( 'v_author, COUNT(*) AS count', 'videobox_videos', $where, NULL, NULL, 'v_author' )->setKeyField('v_author')->setValueField('count') );
						
			/* Get member data and put it together */
			${$time} = array();
			foreach( \IPS\Db::i()->select( '*', 'core_members', \IPS\Db::i()->in( 'member_id', array_keys( $ratings ) ) ) as $key => $memberData )
			{
				${$time}[$key]['member'] = \IPS\Member::constructFromData( $memberData );
				${$time}[$key]['videos']  = isset( $videoCounts[ $memberData['member_id'] ] ) ? $videoCounts[ $memberData['member_id'] ] : 0;
				${$time}[$key]['rating'] = $ratings[ $memberData['member_id'] ];
			}
		}		

		return $this->output( $week, $month, $year, $all );
	}
}