<?php
/**
 * @brief		VideoCollections Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		05 Apr 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * VideoCollections Widget
 */
class _VideoCollections extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'VideoCollections';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
		$this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), 'Videos' ) );
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
		{
	 		$form = new \IPS\Helpers\Form;
 		}

 		$form->add( new \IPS\Helpers\Form\Text( 'vbox_block_title', $this->configuration['vbox_block_title'], FALSE ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_block_theme', isset( $this->configuration['vbox_block_theme'] ) ? $this->configuration['vbox_block_theme'] : 'grid', FALSE, array(
            'options' => array(
				'default'		=> 'vbox_block_theme_default',	
	            'gridview1'		=> 'vbox_block_theme_gridview1',
	            'gridview2'		=> 'vbox_block_theme_gridview2',
				'slider'		=> 'vbox_block_theme_slider',
				//'magicwall'    	=> 'vbox_block_theme_wall',
            )
        ) ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'number_to_show', isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 18, TRUE ) );
		
		$form->add( new \IPS\Helpers\Form\Node( 'vbox_block_collection', isset( $this->configuration['vbox_block_collection'] ) ? $this->configuration['vbox_block_collection'] : 0, TRUE, array(
			'class'           => '\IPS\videobox\Collections\Collection',
			'multiple'        => true
		) ) );			
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_sort_on', isset( $this->configuration['vbox_sort_on'] ) ? $this->configuration['vbox_sort_on'] : 'v_updated', FALSE, array(
            'options' => array(
	            'v_started'     	=> 'vbox_sort_on_started',
				'v_updated'			=> 'vbox_sort_on_updated',
	            'v_title'    		=> 'vbox_sort_on_title',
	            'v_rating_average'	=> 'vbox_sort_on_rating',
	            'v_comments'  		=> 'vbox_sort_on_comments',
				'v_last_comment'	=> 'vbox_sort_on_last_comment',
	            'views'       		=> 'vbox_sort_on_views'
            )
        ) ) );

		$form->add( new \IPS\Helpers\Form\Select( 'vbox_sort_dir', isset( $this->configuration['vbox_sort_dir'] ) ? $this->configuration['vbox_sort_dir'] : 'desc', FALSE, array(
            'options' => array(
	            'desc'   => 'vbox_sort_dir_dsc',
	            'asc'    => 'vbox_sort_dir_asc'
            )
        ) ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vbox_lightbox', $this->configuration['vbox_lightbox'], FALSE, array( 'togglesOn' => array( 'vbox_lightbox_device' ) ) ) );
		//$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vbox_lightbox_device', isset( $this->configuration['vbox_lightbox_device'] ) ? $this->configuration['vbox_lightbox_device'] : FALSE, FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'vbox_lightbox_device' ) );
			
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		if ( \is_array( $values['vbox_block_collection'] ) )
 		{
	 		$values['vbox_block_collection'] = array_keys( $values['vbox_block_collection'] );
 		}
 		
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( \count( $this->configuration['vbox_block_collection'] ) == 0 )
		{
			return "";
		}
		
		$order = NULL;		
		$limit = $this->configuration['number_to_show'] ? $this->configuration['number_to_show'] : 18;
	
		if ( !empty( $this->configuration['vbox_block_collection'] ) )
		{
			foreach( $this->configuration['vbox_block_collection'] as $collectionID )
			{
				$cond[] = "find_in_set('{$collectionID}', v_collections)";
			}			
		}		

		if ( \count( $cond ) > 0 )
		{
			$where[] = array( implode(" OR ", $cond) . " AND v_collections != '' AND v_collections IS NOT NULL" );
		}
		
		if ( isset( $this->configuration['vbox_sort_on'] ) and isset( $this->configuration['vbox_sort_dir'] ) )
		{
			$order = $this->configuration['vbox_sort_on'] . ' ' . $this->configuration['vbox_sort_dir'];
		}		
		
		$videos = \IPS\videobox\Video::getItemsWithPermission( $where, $order, $limit );

		if ( \count( $videos ) )
		{			
			return $this->output( $videos, $this->configuration['vbox_block_title'], $this->configuration, $this->orientation );
		}
		else
		{
			return '';
		}
	}
}