<?php
/**
 * @brief		Videos Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		10 Mar 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Videos Widget
 */
class _Videos extends \IPS\Content\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'Videos';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Class
	 */
	protected static $class = 'IPS\videobox\Video';
	
	public $neverCache = TRUE;
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		$form = parent::configuration( $form );

		if ( \IPS\Dispatcher::i()->controllerLocation != 'admin' )
		{
			unset($form->elements[""]['widget_feed_min_rating']);
			
			$form->elements[""]['widget_feed_container_videobox_video'] = null;

			$form->elements[""]['widget_feed_container_videobox_video'] = new \IPS\Helpers\Form\Node( 'widget_feed_container_videobox_video', isset( $this->configuration['widget_feed_container'] ) ? $this->configuration['widget_feed_container'] : 0, FALSE, array(
				'class'           => '\IPS\videobox\Category',
				'zeroVal'         => 'all',
				'permissionCheck' => 'view',
				'multiple'        => true,
				'forceOwner'	  => false,
				'clubs'			  => true,
			) );
		}
		else
		{
			unset($form->elements["content_block_form_tab__feed"]['widget_feed_min_rating']);
			
			$form->elements["content_block_form_tab__feed"]['widget_feed_container_videobox_video'] = null;

			$form->elements["content_block_form_tab__feed"]['widget_feed_container_videobox_video'] = new \IPS\Helpers\Form\Node( 'widget_feed_container_videobox_video', isset( $this->configuration['widget_feed_container'] ) ? $this->configuration['widget_feed_container'] : 0, FALSE, array(
				'class'           => '\IPS\videobox\Category',
				'zeroVal'         => 'all',
				'permissionCheck' => 'view',
				'multiple'        => true,
				'forceOwner'	  => false,
				'clubs'			  => true,
			) );
		}

		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vbox_lstream_status', isset( $this->configuration['vbox_lstream_status'] ) ? $this->configuration['vbox_lstream_status'] : array(), FALSE, array(
			'options' => array(
				'online'       => 'vbox_lstream_online',
			    'offline'      => 'vbox_lstream_offline'
			)
		) ) );		
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vbox_field_on', $this->configuration['vbox_field_on'], FALSE, array( 'togglesOn' => array( 'vbox_field', 'vbox_field_cond', 'vbox_field_val' ) ) ) );
		
		$customFields = array();
		
		foreach ( \IPS\Db::i()->select( 'cf_id', 'videobox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "videobox_field_{$fieldId}" );
		}	
		
		if ( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'vbox_field', $this->configuration['vbox_field'], FALSE, array( 'options' => $customFields, 'multiple' => FALSE), NULL, NULL, NULL, 'vbox_field' ) );
		}
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_field_cond', $this->configuration['vbox_field_cond'] ? $this->configuration['vbox_field_cond'] : 'equal', TRUE, array(
            'options' => array(
	            'like'   	=> 'v_qs_mode_like',			
	            'equal'   	=> 'v_qs_mode_equal',
	            'greater'   => 'v_qs_mode_greater',
	            'lessthan'  => 'v_qs_mode_lessthan',
	            'different'	=> 'v_qs_mode_different',					
            )
        ), NULL, NULL, NULL, 'vbox_field_cond' ) );
		
		$form->add( new \IPS\Helpers\Form\Text( 'vbox_field_val', $this->configuration['vbox_field_val'], FALSE, array(), NULL, NULL, NULL, 'vbox_field_cond' ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_block_theme', $this->configuration['vbox_block_theme'] ? $this->configuration['vbox_block_theme'] : 'default', FALSE, array(
            'options' => array(
				'default'		=> 'vbox_block_theme_default',			
	            'gridview1'    	=> 'vbox_block_theme_gridview1',
	            'gridview2'    	=> 'vbox_block_theme_gridview2',
	            'slider'    	=> 'vbox_block_theme_slider',
	            //'magicwall'    	=> 'vbox_block_theme_wall',
            )
        ) ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vbox_lightbox', $this->configuration['vbox_lightbox'], FALSE, array( 'togglesOn' => array( 'vbox_lightbox_device' ) ) ) );
		//$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vbox_lightbox_device', isset( $this->configuration['vbox_lightbox_device'] ) ? $this->configuration['vbox_lightbox_device'] : FALSE, FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'vbox_lightbox_device' ) );
			
		return $form;
 	} 
	
	
	/**
	 * Get where clause
	 *
	 * @return	array
	 */
	protected function buildWhere()
	{
		$where = parent::buildWhere();
		
		foreach( $where as $k => $vals )
		{
			if ( \is_array( $vals ) )
			{
				foreach( $vals as $v )
				{
					if ( $v == "videobox_videos.v_started>?" )
					{
						unset( $where[$k] );
					}					
				}
			}
		}

		if ( $this->configuration['widget_feed_restrict_days'] != -1 )
		{
			if ( isset( $this->configuration['widget_feed_restrict_days'] ) and $this->configuration['widget_feed_restrict_days'] > 0 )
			{
				$where[] = array(  'videobox_videos.v_started>?',  \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $this->configuration['widget_feed_restrict_days'] . 'D' ) )->getTimestamp() );
			}
		}
		if ( !empty( $this->configuration['vbox_lstream_status'] ) )
		{
			$status = $this->configuration['vbox_lstream_status'];

			if ( \in_array( 'online', $status ) and ! \in_array( 'offline', $status ) )
			{
				$where[] = array( "v_livestream_online='online'" );
			}
			
			if ( \in_array( 'offline', $status ) and ! \in_array( 'online', $status ) )
			{
				$where[] = array( "v_livestream_online='offline'" );
			}		
		}

		if ( $this->configuration['vbox_field_on'] == 1 )
		{
			try
			{
				$field = \IPS\videobox\Field::load( $this->configuration['vbox_field'] );
			}
			catch ( \OutOfRangeException $e ) {}

			if ( $field )
			{		
				# Search term
				$searchterm = mb_strtolower( $this->configuration['vbox_field_val'] );

				if ( $field->type == 'CheckboxSet' )
				{
					foreach( json_decode($field->content) as $k => $val )
					{
						if ( mb_strtolower($val) == $searchterm )
						{
							$query = "find_in_set('{$k}', field_{$field->field})";
							break;
						}
					}
				}
				else
				{
					switch ( $this->configuration['vbox_field_cond'] )
					{
						case 'like':
							$query = "field_{$field->id} LIKE '%" . $searchterm . "%'";
							break;				
						case 'equal':
							$query = "field_{$field->id}='" . $searchterm . "'";
							break;
						case 'greater':
							$query = "field_{$field->id}>='" . $searchterm . "'";
							break;
						case 'lessthan':
							$query = "field_{$field->id}<='" . $searchterm . "'";					
							break;
						case 'different':
							$query = "field_{$field->id}!='" . $searchterm . "'";					
							break;			
					}	
				}
				foreach ( \IPS\Db::i()->select( "lower(field_{$field->id}), video_id", "videobox_ccontent", $query, $order, array(0, $limit) ) as $v )
				{
					$tid[] = $v['video_id'];
				}
				
				$where[] = \count( $tid ) > 0 ? array( "v_id IN (" . implode( ',', $tid ) . ")" ) : array( "v_id=0" );
			}
		}
		
		return $where;
	}	
	
	/**
	 * Get HTML using the template (language strings not parsed)
	 *
	 * @return	string
	 */	
	public function output()
	{
		$args = \func_get_args();

		$args['conf'] = $this->configuration;
		$args['orientation'] = $this->orientation;
		
		$output = \call_user_func_array( $this->template, $args );

		return $output;
	}
}