<?php
/**
 * @brief		videoboxTop Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	videobox
 * @since		04 Jan 2018
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * videoboxTop Widget
 */
class _videoboxTop extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'videoboxTop';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		// Use this to perform any set up and to assign a template that is not in the following format:
		// $this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
		// If you are creating a plugin, uncomment this line:
		// $this->template( array( \IPS\Theme::i()->getTemplate( 'plugins', 'core', 'global' ), $this->key ) );
		// And then create your template at located at plugins/<your plugin>/dev/html/videoboxTop.phtml
		
		
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
		{
	 		$form = new \IPS\Helpers\Form;
 		}
		
		$form->add( new \IPS\Helpers\Form\Translatable( 'widget_feed_title', isset( $this->configuration['language_key'] ) ? NULL : \IPS\Member::loggedIn()->language()->addToStack( \IPS\videobox\Video::$title . '_pl' ), FALSE, array( 'app' => 'videobox', 'key' => ( isset( $this->configuration['language_key'] ) ? $this->configuration['language_key'] : NULL ) ) ) );

		$form->add( new \IPS\Helpers\Form\Select( 'vb_widget_feed_type', $this->configuration['vb_widget_feed_type'] ? $this->configuration['vb_widget_feed_type'] : 'v_views', FALSE, array( 
			'options' => array(
				'v_views'			=> 'vb_hot_views',
				'v_reviews'			=> 'vb_hot_reviews',
				'v_rating_average'	=> 'vb_hot_average',
			)
		) ) );
		
		
		$form->add( new \IPS\Helpers\Form\Node( 'widget_feed_container_' . \IPS\videobox\Video::$title, isset( $this->configuration['widget_feed_container'] ) ? $this->configuration['widget_feed_container'] : 0, FALSE, array(
			'class'           => 'IPS\videobox\Category',
			'zeroVal'         => 'all',
			'permissionCheck' => 'view',
			'multiple'        => true,
			'forceOwner'	  => false,
			'clubs'			  => true,
		) ) );		
		
		$form->add( new \IPS\Helpers\Form\Number( 'number_to_show', isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 5, TRUE ) );

		$form->add( new \IPS\Helpers\Form\YesNo( 'vbox_lightbox', $this->configuration['vbox_lightbox'], FALSE, array( 'togglesOn' => array( 'vbox_lightbox_device' ) ) ) );
		//$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vbox_lightbox_device', isset( $this->configuration['vbox_lightbox_device'] ) ? $this->configuration['vbox_lightbox_device'] : FALSE, FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'vbox_lightbox_device' ) );
			
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		if ( \is_array( $values[ 'widget_feed_container_' . \IPS\videobox\Video::$title ] ) )
 		{
	 		$values['widget_feed_container'] = array_keys( $values[ 'widget_feed_container_' . \IPS\videobox\Video::$title ] );
			unset( $values[ 'widget_feed_container_' . \IPS\videobox\Video::$title ] );
 		}
		
 		if ( !isset( $this->configuration['language_key'] ) )
 		{
	 		$this->configuration['language_key'] = 'widget_title_' . md5( uniqid() );
 		}
		$values['language_key'] = $this->configuration['language_key'];

 		\IPS\Lang::saveCustom( 'videobox', $this->configuration['language_key'], $values['widget_feed_title'] );
 		unset( $values['widget_feed_title'] );		
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		$limit = isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 5;
	
		$categories = array();

		foreach( \IPS\Db::i()->select( 'perm_type_id', 'core_permission_index', array( 'app=? and perm_type=? and (' . \IPS\Db::i()->findInSet( 'perm_' . \IPS\videobox\Category::$permissionMap['read'], \IPS\Member::loggedIn()->groups ) . ' OR ' . 'perm_' . \IPS\videobox\Category::$permissionMap['read'] . '=? )', 'videobox', 'category', '*' ) ) as $category )
		{
			if ( !empty( $this->configuration['widget_feed_container'] ) )
			{
				if ( \in_array($category, $this->configuration['widget_feed_container']) )
				{
					$categories[] = $category;
				}
			}
			else
			{
				$categories[]	= $category;
			}
		}

		if( !\count( $categories ) )
		{
			return '';
		}

		$type = $this->configuration['vb_widget_feed_type'] ? $this->configuration['vb_widget_feed_type'] : 'v_views';
		
		$order = $type . " DESC";

		foreach ( array( 'week' => 'P1W', 'month' => 'P1M', 'year' => 'P1Y', 'all' => NULL ) as $time => $interval )
		{			
			$where = array( array( 'v_cat IN(' . implode( ',', $categories ) . ')' ) );
			
			if ( $interval )
			{
				$where[] = array( 'v_started>?', \IPS\DateTime::create()->sub( new \DateInterval( $interval ) )->getTimestamp() );
			}
			
			$$time = new \IPS\Patterns\ActiveRecordIterator(
				\IPS\Db::i()->select( '*', 'videobox_videos', $where, $order, $limit ), 'IPS\videobox\Video'
			);
		}
		

		if ( isset( $this->configuration['language_key'] ) )
		{
			$title = \IPS\Member::loggedIn()->language()->addToStack( $this->configuration['language_key'], FALSE, array( 'escape' => TRUE ) );
		}
		elseif ( isset( $this->configuration['widget_feed_title'] ) )
		{
			$title = $this->configuration['widget_feed_title'];
		}
		else
		{
			$title = \IPS\Member::loggedIn()->language()->addToStack( \IPS\videobox\Video::$title . '_pl' );
		}	
		
			
		return $this->output( $title, $this->configuration, $week, $month, $year, $all );
	}
}